# -*- coding: utf-8 -*-
#
#       ramstk.models.hardware.view.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Hardware Package View Model."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub
from sqlalchemy.orm.exc import ObjectDeletedError
from treelib import Tree

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseView


class RAMSTKHardwareBoMView(RAMSTKBaseView):
    """Contain the attributes and methods of the Hardware BoM view.

    This class manages the hardware BoM data from the RAMSTKHardware,
    RAMSTKDesignElectric, RAMSTKDesignMechanic, RAMSTKMilHdbkF, RAMSTKNSWC, and
    RAMSTKReliability table models.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag = "hardware_bom"

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self, **kwargs: Dict[Any, Any]) -> None:
        """Initialize a Hardware BoM view model instance."""
        super().__init__(**kwargs)

        # Initialize private dictionary attributes.
        self._dic_load_functions = {
            "hardware": self._do_load_hardware,
            "design_electric": self._do_load_design_electric,
            "design_mechanic": self._do_load_design_mechanic,
            "milhdbk217f": self._do_load_milhdbk217f,
            "nswc": self._do_load_nswc,
            "reliability": self._do_load_reliability,
        }
        self._dic_stress_limits: Dict[int, List[float]] = kwargs.get(
            "stress_limits",
            {
                1: [
                    0.8,
                    0.9,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                ],
                2: [
                    1.0,
                    1.0,
                    0.7,
                    0.9,
                    1.0,
                    1.0,
                ],
                3: [
                    1.0,
                    1.0,
                    0.5,
                    0.9,
                    1.0,
                    1.0,
                ],
                4: [
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                    0.6,
                    0.9,
                ],
                5: [
                    0.6,
                    0.9,
                    1.0,
                    1.0,
                    0.5,
                    0.9,
                ],
                6: [
                    0.75,
                    0.9,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                ],
                7: [
                    0.75,
                    0.9,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                ],
                8: [
                    0.7,
                    0.9,
                    1.0,
                    1.0,
                    0.7,
                    0.9,
                ],
                9: [
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                ],
                10: [
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                    1.0,
                ],
            },
        )
        self._dic_trees = {
            "hardware": Tree(),
            "design_electric": Tree(),
            "design_mechanic": Tree(),
            "milhdbk217f": Tree(),
            "nswc": Tree(),
            "reliability": Tree(),
        }

        # Initialize private list attributes.
        self._lst_modules = [
            "hardware",
            "design_electric",
            "design_mechanic",
            "milhdbk217f",
            "nswc",
            "reliability",
        ]

        # Initialize private scalar attributes.
        self._hr_multiplier = kwargs.get("hr_multiplier", 1.0)

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_set_tree, "succeed_insert_hardware")
        pub.subscribe(super().do_set_tree, "succeed_insert_design_electric")
        pub.subscribe(super().do_set_tree, "succeed_insert_design_mechanic")
        pub.subscribe(super().do_set_tree, "succeed_insert_milhdbk217f")
        pub.subscribe(super().do_set_tree, "succeed_insert_nswc")
        pub.subscribe(super().do_set_tree, "succeed_insert_reliability")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_hardwares")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_design_electrics")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_design_mechanics")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_milhdbk217fs")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_nswcs")
        pub.subscribe(super().do_set_tree, "succeed_retrieve_reliabilitys")
        pub.subscribe(super().do_set_tree, "succeed_delete_hardware")
        pub.subscribe(super().do_set_tree, "succeed_delete_design_electric")
        pub.subscribe(super().do_set_tree, "succeed_delete_design_mechanic")
        pub.subscribe(super().do_set_tree, "succeed_delete_milhdbk217f")
        pub.subscribe(super().do_set_tree, "succeed_delete_nswc")
        pub.subscribe(super().do_set_tree, "succeed_delete_reliability")
        pub.subscribe(self.do_calculate_hardware, "request_calculate_hardware")
        pub.subscribe(self.do_make_composite_ref_des, "request_make_comp_ref_des")

    def do_calculate_cost(self, node_id: int) -> None:
        """Calculate the cost related metrics.

        :param node_id: the record ID to calculate.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data["hardware"]
        _total_cost: float = 0.0

        if _record.part == 1:
            _record.do_calculate_total_cost()
        else:
            for _node in self.tree.children(node_id):
                self.do_calculate_cost(_node.identifier)
                _total_cost += _node.data["hardware"].total_cost
                _total_cost = _total_cost * _record.quantity
            _record.set_attributes({"total_cost": _total_cost})

    def do_calculate_hardware(self, node_id: int) -> None:
        """Calculate all metrics for the hardware associated with node ID.

        :param node_id: the record ID to calculate.
        :return: None
        :rtype: None
        """
        # ISSUE: Assembly hazard rates not rolling up
        #
        # When calculating assemblies, the hazard rate, MTBF, reliability,
        # etc. metrics are not being calculated for each assembly.
        #
        # priority: high
        # bump: patch
        _record = self.tree.get_node(node_id)
        _attributes = {
            **_record.data["hardware"].get_attributes(),
            **_record.data["design_mechanic"].get_attributes(),
            **_record.data["design_electric"].get_attributes(),
            **_record.data["milhdbk217f"].get_attributes(),
            **_record.data["nswc"].get_attributes(),
            **_record.data["reliability"].get_attributes(),
        }

        self.do_calculate_cost(node_id)
        self.do_calculate_part_count(node_id)
        self.do_calculate_power_dissipation(node_id)
        if _attributes["part"] == 1:
            _record.data["design_electric"].do_stress_analysis(
                _attributes["category_id"]
            )
            _record.data["design_electric"].do_derating_analysis(
                self._dic_stress_limits[_attributes["category_id"]]
            )

        _record.data["reliability"].do_calculate_hazard_rate_active(
            self._hr_multiplier,
            _attributes,
            time=_attributes["mission_time"],
        )

        if _attributes["part"] == 1:
            _record.data["reliability"].do_calculate_hazard_rate_dormant(
                _attributes["category_id"],
                _attributes["subcategory_id"],
                _attributes["environment_active_id"],
                _attributes["environment_dormant_id"],
            )
        _record.data["reliability"].do_calculate_hazard_rate_logistics()
        _record.data["reliability"].do_calculate_hazard_rate_mission(
            _attributes["duty_cycle"]
        )
        _record.data["reliability"].do_calculate_mtbf()
        _record.data["reliability"].do_calculate_reliability(
            _attributes["mission_time"]
        )

        pub.sendMessage(
            "request_get_milhdbk217f_attributes",
            node_id=node_id,
        )
        pub.sendMessage(
            "request_get_reliability_attributes",
            node_id=node_id,
        )

    def do_calculate_part_count(self, node_id: int) -> None:
        """Calculate the total part count of a hardware item.

        :param node_id: the record ID to calculate.
        :return: None
        :rtype: None
        """
        _record = self.tree.get_node(node_id).data["hardware"]
        _total_part_count: int = 0

        if _record.part == 1:
            _record.total_part_count = _record.quantity
        else:
            for _node in self.tree.children(node_id):
                self.do_calculate_part_count(_node.identifier)
                _total_part_count += _node.data["hardware"].total_part_count
                _total_part_count = _total_part_count * _record.quantity

            _record.set_attributes({"total_part_count": _total_part_count})

    def do_calculate_power_dissipation(self, node_id: int) -> float:
        """Calculate the total power dissipation of a hardware item.

        :param node_id: the record ID to calculate.
        :return: _total_power_dissipation; the total power dissipation.
        :rtype: float
        """
        _record = self.tree.get_node(node_id)
        _total_power_dissipation: float = 0.0

        if _record.data["hardware"].part == 1:
            _total_power_dissipation = (
                _record.data["design_electric"].power_operating
                * _record.data["hardware"].quantity
            )
        else:
            for _node_id in _record.successors(self.tree.identifier):
                _total_power_dissipation += self.do_calculate_power_dissipation(
                    _node_id
                )
            _total_power_dissipation = (
                _total_power_dissipation * _record.data["hardware"].quantity
            )

        _record.data["hardware"].set_attributes(
            {"total_power_dissipation": _total_power_dissipation}
        )

        return _total_power_dissipation

    def do_make_composite_ref_des(self, node_id: int = 1) -> None:
        """Make the composite reference designators.

        :param node_id: the record ID to start making the composite reference
            designators.
        :return: None
        :rtype: None
        """
        # Retrieve the parent hardware item's composite reference designator.
        _node = self.tree.get_node(node_id)
        _record = _node.data["hardware"]

        if self.tree.parent(node_id).identifier != 0:
            _p_comp_ref_des = self.tree.parent(node_id).data["hardware"].comp_ref_des
        else:
            _p_comp_ref_des = ""

        if _p_comp_ref_des != "":
            _record.comp_ref_des = _p_comp_ref_des + ":" + _record.ref_des
            _node.tag = _p_comp_ref_des + ":" + _record.ref_des
        else:
            _record.comp_ref_des = _record.ref_des
            _node.tag = _record.ref_des

        # Now make the composite reference designator for all the child nodes.
        for _child_node in self.tree.children(node_id):
            self.do_make_composite_ref_des(node_id=_child_node.identifier)

        _record.set_attributes({"comp_ref_des": _record.comp_ref_des})

        pub.sendMessage(
            "succeed_make_comp_ref_des",
            comp_ref_des=_record.comp_ref_des,
        )

    def _do_load_hardware(self) -> None:
        """Load the hardware data into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["hardware"].all_nodes()[1:]:
            _hardware = _node.data["hardware"]

            self.tree.create_node(
                tag="hardware",
                identifier=_hardware.hardware_id,
                parent=_hardware.parent_id,
                data={"hardware": _hardware},
            )

        self._dic_load_functions["design_electric"]()
        self._dic_load_functions["design_mechanic"]()
        self._dic_load_functions["milhdbk217f"]()
        self._dic_load_functions["nswc"]()
        self._dic_load_functions["reliability"]()

    def _do_load_design_electric(self) -> None:
        """Load the design electric data into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["design_electric"].all_nodes()[1:]:
            _design_electric = _node.data["design_electric"]

            try:
                _par_node = self.tree.get_node(_design_electric.hardware_id)
                _par_node.data["design_electric"] = _design_electric
            except ObjectDeletedError:
                self._dic_trees["design_electric"].remove_node(_node.identifier)

    def _do_load_design_mechanic(self) -> None:
        """Load the design_mechanic into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["design_mechanic"].all_nodes()[1:]:
            _design_mechanic = _node.data["design_mechanic"]

            try:
                _par_node = self.tree.get_node(_design_mechanic.hardware_id)
                _par_node.data["design_mechanic"] = _design_mechanic
            except ObjectDeletedError:
                self._dic_trees["design_mechanic"].remove_node(_node.identifier)

    def _do_load_milhdbk217f(self) -> None:
        """Load the MIL-HDBK-217F data into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["milhdbk217f"].all_nodes()[1:]:
            _milhdbk217f = _node.data["milhdbk217f"]

            try:
                _par_node = self.tree.get_node(_milhdbk217f.hardware_id)
                _par_node.data["milhdbk217f"] = _milhdbk217f
            except ObjectDeletedError:
                self._dic_trees["milhdbk217f"].remove_node(_node.identifier)

    def _do_load_nswc(self) -> None:
        """Load the NSWC data into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["nswc"].all_nodes()[1:]:
            _nswc = _node.data["nswc"]

            try:
                _par_node = self.tree.get_node(_nswc.hardware_id)
                _par_node.data["nswc"] = _nswc
            except ObjectDeletedError:
                self._dic_trees["nswc"].remove_node(_node.identifier)

    def _do_load_reliability(self) -> None:
        """Load the reliability data into the tree.

        :return: None
        :rtype: None
        """
        for _node in self._dic_trees["reliability"].all_nodes()[1:]:
            _reliability = _node.data["reliability"]

            try:
                _par_node = self.tree.get_node(_reliability.hardware_id)
                _par_node.data["reliability"] = _reliability
            except ObjectDeletedError:
                self._dic_trees["reliability"].remove_node(_node.identifier)



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.pof.view.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 PoF Views."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.assistants import AddStressTestMethod
from ramstk.views.gtk3.widgets import RAMSTKMessageDialog, RAMSTKPanel, RAMSTKWorkView

# RAMSTK Local Imports
from . import PoFTreePanel


class PoFWorkView(RAMSTKWorkView):
    """Display PoF attribute data in the Work Book.

    The WorkView displays all the attributes for the Physics of Failure
    Analysis (PoF). The attributes of a PoF Work View are:

    :cvar dict _dic_column_masks: dict with the list of masking values for
        the PoF worksheet.  Key is the PoF indenture level, value is a
        list of True/False values for each column in the worksheet.
    :cvar dict _dic_headings: the dict with the variable headings for the
        first two columns.  Key is the name of the PoF indenture level,
        value is a list of heading text.
    :cvar dict _dic_keys:
    :cvar dict _dic_column_keys:
    :cvar list _lst_labels: the list of labels for the widgets on the work
        view.  The PoF work stream module has no labels, but an empty list
        is required to prevent an AttributeError when creating the UI.
    :cvar str _tag: the name of the module.
    :cvar bool _pixbuf: indicates whether or icons are displayed in the
        RAMSTKTreeView.  If true, a GDKPixbuf column will be appended when
        creating the RAMSTKTreeView.  Default is True.

    :ivar list _lst_callbacks: the list of callback methods for the view's
        toolbar buttons and pop-up menu.  The methods are listed in the order
        they appear on the toolbar and pop-up menu.
    :ivar list _lst_icons: the list of icons for the view's toolbar buttons
        and pop-up menu.  The icons are listed in the order they appear on the
        toolbar and pop-up menu.
    :ivar list _lst_mnu_labels: the list of labels for the view's pop-up
        menu.  The labels are listed in the order they appear in the menu.
    :ivar list _lst_tooltips: the list of tooltips for the view's
        toolbar buttons and pop-up menu.  The tooltips are listed in the
        order they appear on the toolbar or pop-up menu.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _tag: str = "pof"
    _pixbuf: bool = True
    _tablabel = _("PoF")
    _tabtooltip = _(
        "Displays the Physics of Failure (PoF) Analysis for "
        "the selected hardware item."
    )

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None:
        """Initialize the Work View for the PoF.

        :param configuration: the RAMSTKUserConfiguration class instance.
        :param logger: the RAMSTKLogManager class instance.
        """
        super().__init__(configuration, logger)

        # Initialize private dict attributes.

        # Initialize private list attributes.
        self._lst_callbacks.insert(0, self._do_request_insert_sibling)
        self._lst_callbacks.insert(1, self._do_request_insert_child)
        self._lst_callbacks.insert(2, self._do_request_delete)
        self._lst_icons.insert(0, "insert_sibling")
        self._lst_icons.insert(1, "insert_child")
        self._lst_icons.insert(2, "remove")
        self._lst_mnu_labels.insert(0, _("Insert Sibling"))
        self._lst_mnu_labels.insert(1, _("Insert Child"))
        self._lst_mnu_labels.insert(2, _("Delete Selected"))
        self._lst_tooltips: List[str] = [
            _(
                "Add a new PoF entity at the same level as the currently selected "
                "entity."
            ),
            _("Add a new PoF entity one level below the currently selected entity."),
            _("Remove the selected entity from the PoF."),
            _("Save changes to the currently selected PoF line."),
            _("Save changes to all PoF lines."),
        ]

        # Initialize private scalar attributes.
        self._pnlPanel: RAMSTKPanel = PoFTreePanel()

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_set_record_id, "selected_pof")

    def _do_request_delete(self, __button: Gtk.ToolButton) -> None:
        """Request to delete the selected entity from the PoF.

        :param __button: the Gtk.ToolButton() that called this method.
        :return: None
        :rtype: None
        """
        _parent = self.get_parent().get_parent().get_parent().get_parent()
        _model, _row = self._pnlPanel.tvwTreeView.get_selection().get_selected()
        _node_id = _model.get_value(_row, 0)

        _prompt = _(
            "You are about to delete {1} item {0} and all "
            "data associated with it.  Is this really what "
            "you want to do?"
        ).format(_node_id, self._tag.title())
        _dialog = RAMSTKMessageDialog(parent=_parent)
        _dialog.do_set_message(_prompt)
        _dialog.do_set_message_type("question")

        if _dialog.do_run() == Gtk.ResponseType.YES:
            super().do_set_cursor_busy()
            pub.sendMessage(
                "request_delete_pof",
                node_id=_node_id,
            )

        _dialog.do_destroy()

    def _do_request_insert_child(self, __button: Gtk.ToolButton) -> None:
        """Request to insert a new child entity to the PoF.

        :return: None
        :rtype: None
        """
        # Try to get the information needed to add a new entity at the correct
        # location in the PoF.
        _model, _row = self._pnlPanel.tvwTreeView.get_selection().get_selected()
        try:
            _parent_id = _model.get_value(_row, 0)
            _level = {
                2: "opload",
                3: "opstress_testmethod",
            }[len(str(_parent_id).split("."))]
        except TypeError:
            _parent_id = "0"
            _level = "opload"

        if _level == "opstress_testmethod":
            _level = self.__on_request_insert_opstress_method()

        super().do_set_cursor_busy()
        pub.sendMessage(f"request_insert_pof_{_level}", parent_id=str(_parent_id))

    def _do_request_insert_sibling(self, __button: Gtk.ToolButton) -> None:
        """Request to insert a new sibling entity to the PoF.

        :return: None
        :rtype: None
        """
        # Try to get the information needed to add a new entity at the correct
        # location in the PoF.
        _model, _row = self._pnlPanel.tvwTreeView.get_selection().get_selected()
        try:
            _parent_id = _model.get_value(_model.iter_parent(_row), 0)
            _level = {
                2: "opload",
                3: "opstress_testmethod",
            }[len(str(_parent_id).split("."))]
        except TypeError:
            _parent_id = "0"
            _level = "opload"

        if _level == "opstress_testmethod":
            _level = self.__on_request_insert_opstress_method()

        super().do_set_cursor_busy()
        pub.sendMessage(f"request_insert_pof_{_level}", parent_id=str(_parent_id))

    def _do_set_record_id(self, attributes: Dict[str, Any]) -> None:
        """Set the record and revision ID when a hardware item is selected.

        :param attributes: the hazard dict for the selected hardware ID.
        :return: None
        :rtype: None
        """
        self.dic_pkeys["record_id"] = attributes["node_id"]

    def __do_load_test_method_lists(self):
        """Load the Gtk.CellRendererCombo()s associated with test methods.

        :return: None
        :rtype: None
        """
        self._pnlPanel.lst_damage_models = [
            x[1] for x in self.RAMSTK_USER_CONFIGURATION.RAMSTK_DAMAGE_MODELS.items()
        ]
        self._pnlPanel.lst_load_history = [
            x[1] for x in self.RAMSTK_USER_CONFIGURATION.RAMSTK_LOAD_HISTORY.items()
        ]
        self._pnlPanel.lst_measurable_parameters = [
            x[1][1]
            for x in self.RAMSTK_USER_CONFIGURATION.RAMSTK_MEASURABLE_PARAMETERS.items()
        ]

    def __make_ui(self) -> None:
        """Build the user interface for the PoF tab.

        :return: None
        :rtype: None
        """
        super().do_make_layout()

        self._pnlPanel.dic_icons = self._dic_icons

        super().do_embed_treeview_panel()
        self.__do_load_test_method_lists()
        self._pnlPanel.do_load_comboboxes()

        self.show_all()

    def __on_request_insert_opstress_method(self) -> str:
        """Raise dialog to select whether to add a stress or test method.

        :return: _level; the level to add, opstress or testmethod.
        :rtype: str
        """
        _level = ""

        _dialog = AddStressTestMethod(
            parent=self.get_parent().get_parent().get_parent().get_parent()
        )

        if _dialog.do_run() == Gtk.ResponseType.OK:
            if _dialog.rdoOpStress.get_active():
                _level = "opstress"
            elif _dialog.rdoTestMethod.get_active():
                _level = "testmethod"

        _dialog.do_destroy()

        return _level



# pylint: disable=duplicate-code
# -*- coding: utf-8 -*-
#
#       ramstk.models.revision.record.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Revision Record Model."""

# Third Party Imports
from sqlalchemy import Column, Float, Integer, String
from sqlalchemy.orm import relationship

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKRevisionRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_revision table in the RAMSTK Program database.

    This table shares a:
        * One-to-Many relationship with ramstk_environment.
        * One-to-Many relationship with ramstk_mission.
        * One-to-Many relationship with ramstk_failure_definition.
        * One-to-Many relationship with ramstk_function.
        * One-to-Many relationship with ramstk_requirement.
        * One-to-Many relationship with ramstk_hardware.
        * One-to-Many relationship with ramstk_software.
        * One-to-Many relationship with ramstk_validation.
        * One-to-Many relationship with ramstk_incident.
        * One-to-Many relationship with ramstk_survival.
        * One-to-Many relationship with ramstk_matrix.
        * One-to-Many relationship with ramstk_hazard_analysis.
        * One-to-Many relationship with ramstk_program_status.
    """

    __defaults__ = {
        "availability_logistics": 1.0,
        "availability_mission": 1.0,
        "cost": 0.0,
        "cost_failure": 0.0,
        "cost_hour": 0.0,
        "hazard_rate_active": 0.0,
        "hazard_rate_dormant": 0.0,
        "hazard_rate_logistics": 0.0,
        "hazard_rate_mission": 0.0,
        "hazard_rate_software": 0.0,
        "mmt": 0.0,
        "mcmt": 0.0,
        "mpmt": 0.0,
        "mtbf_logistics": 0.0,
        "mtbf_mission": 0.0,
        "mttr": 0.0,
        "name": "",
        "reliability_logistics": 1.0,
        "reliability_mission": 1.0,
        "remarks": "",
        "total_part_count": 1,
        "revision_code": "",
        "program_time": 0.0,
        "program_time_sd": 0.0,
        "program_cost": 0.0,
        "program_cost_sd": 0.0,
    }
    __tablename__ = "ramstk_revision"
    __table_args__ = {"extend_existing": True}

    revision_id = Column(
        "fld_revision_id",
        Integer,
        primary_key=True,
        autoincrement=True,
        nullable=False,
    )
    availability_logistics = Column(
        "fld_availability_logistics",
        Float,
        nullable=False,
        default=__defaults__["availability_logistics"],
    )
    availability_mission = Column(
        "fld_availability_mission",
        Float,
        nullable=False,
        default=__defaults__["availability_mission"],
    )
    cost = Column("fld_cost", Float, nullable=False, default=__defaults__["cost"])
    cost_failure = Column(
        "fld_cost_failure", Float, nullable=False, default=__defaults__["cost_failure"]
    )
    cost_hour = Column(
        "fld_cost_hour", Float, nullable=False, default=__defaults__["cost_hour"]
    )
    hazard_rate_active = Column(
        "fld_hazard_rate_active",
        Float,
        nullable=False,
        default=__defaults__["hazard_rate_active"],
    )
    hazard_rate_dormant = Column(
        "fld_hazard_rate_dormant",
        Float,
        nullable=False,
        default=__defaults__["hazard_rate_dormant"],
    )
    hazard_rate_logistics = Column(
        "fld_hazard_rate_logistics",
        Float,
        nullable=False,
        default=__defaults__["hazard_rate_logistics"],
    )
    hazard_rate_mission = Column(
        "fld_hazard_rate_mission",
        Float,
        nullable=False,
        default=__defaults__["hazard_rate_mission"],
    )
    hazard_rate_software = Column(
        "fld_hazard_rate_software",
        Float,
        nullable=False,
        default=__defaults__["hazard_rate_software"],
    )
    mmt = Column("fld_mmt", Float, nullable=False, default=__defaults__["mmt"])
    mcmt = Column("fld_mcmt", Float, nullable=False, default=__defaults__["mcmt"])
    mpmt = Column("fld_mpmt", Float, nullable=False, default=__defaults__["mpmt"])
    mtbf_logistics = Column(
        "fld_mtbf_logistics",
        Float,
        nullable=False,
        default=__defaults__["mtbf_logistics"],
    )
    mtbf_mission = Column(
        "fld_mtbf_mission", Float, nullable=False, default=__defaults__["mtbf_mission"]
    )
    mttr = Column("fld_mttr", Float, nullable=False, default=__defaults__["mttr"])
    name = Column("fld_name", String(128), nullable=False, default=__defaults__["name"])
    reliability_logistics = Column(
        "fld_reliability_logistics",
        Float,
        nullable=False,
        default=__defaults__["reliability_logistics"],
    )
    reliability_mission = Column(
        "fld_reliability_mission",
        Float,
        nullable=False,
        default=__defaults__["reliability_mission"],
    )
    remarks = Column(
        "fld_remarks", String, nullable=False, default=__defaults__["remarks"]
    )
    total_part_count = Column(
        "fld_total_part_count",
        Integer,
        nullable=False,
        default=__defaults__["total_part_count"],
    )
    revision_code = Column(
        "fld_revision_code",
        String(8),
        nullable=False,
        default=__defaults__["revision_code"],
    )
    program_time = Column(
        "fld_program_time", Float, nullable=False, default=__defaults__["program_time"]
    )
    program_time_sd = Column(
        "fld_program_time_sd",
        Float,
        nullable=False,
        default=__defaults__["program_time_sd"],
    )
    program_cost = Column(
        "fld_program_cost", Float, nullable=False, default=__defaults__["program_cost"]
    )
    program_cost_sd = Column(
        "fld_program_cost_sd",
        Float,
        nullable=False,
        default=__defaults__["program_cost_sd"],
    )

    # Define the relationships to other tables in the RAMSTK Program database.
    failures: relationship = relationship(
        "RAMSTKFailureDefinitionRecord",
        backref="revision",
        passive_deletes=True,
    )
    mission: relationship = relationship(
        "RAMSTKMissionRecord",
        backref="revision",
        passive_deletes=True,
    )
    function: relationship = relationship(
        "RAMSTKFunctionRecord",
        backref="revision",
        passive_deletes=True,
    )
    requirement: relationship = relationship(
        "RAMSTKRequirementRecord",
        backref="revision",
        passive_deletes=True,
    )
    stakeholder: relationship = relationship(
        "RAMSTKStakeholderRecord",
        backref="revision",
        passive_deletes=True,
    )
    hardware: relationship = relationship(
        "RAMSTKHardwareRecord",
        backref="revision",
        passive_deletes=True,
    )
    validation: relationship = relationship(
        "RAMSTKValidationRecord",
        backref="revision",
        passive_deletes=True,
    )
    hazard: relationship = relationship(
        "RAMSTKHazardRecord",
        backref="revision",
        passive_deletes=True,
    )
    program_status: relationship = relationship(
        "RAMSTKProgramStatusRecord",
        backref="revision",
        passive_deletes=True,
    )

    def get_attributes(self):
        """Retrieve current values of the RAMSTKRevision data model attributes.

        :return: {revision_id, availability, mission_availability, cost,
                  cost_per_failure, cost_per_hour, active_hazard_rate,
                  dormant_hazard_rate, mission_hazard_rate, hazard_rate,
                  software_hazard_rate, mmt, mcmt, mpmt, mission_mtbf, mtbf,
                  mttr, name, mission_reliability, reliability, remarks,
                  n_parts, code, program_time, program_time_se, program_cost,
                  program_cost_se} pairs.
        :rtype: dict
        """
        _attributes = {
            "revision_id": self.revision_id,
            "availability_logistics": self.availability_logistics,
            "availability_mission": self.availability_mission,
            "cost": self.cost,
            "cost_failure": self.cost_failure,
            "cost_hour": self.cost_hour,
            "hazard_rate_active": self.hazard_rate_active,
            "hazard_rate_dormant": self.hazard_rate_dormant,
            "hazard_rate_logistics": self.hazard_rate_logistics,
            "hazard_rate_mission": self.hazard_rate_mission,
            "hazard_rate_software": self.hazard_rate_software,
            "mmt": self.mmt,
            "mcmt": self.mcmt,
            "mpmt": self.mpmt,
            "mtbf_logistics": self.mtbf_logistics,
            "mtbf_mission": self.mtbf_mission,
            "mttr": self.mttr,
            "name": self.name,
            "reliability_logistics": self.reliability_logistics,
            "reliability_mission": self.reliability_mission,
            "remarks": self.remarks,
            "total_part_count": self.total_part_count,
            "revision_code": self.revision_code,
            "program_time": self.program_time,
            "program_time_sd": self.program_time_sd,
            "program_cost": self.program_cost,
            "program_cost_sd": self.program_cost_sd,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.analyses.derating.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Component Derating Calculations Module."""


def check_overstress(op_stress, limits):
    """Check if an operating condition results in an overstressed condition.

    Checks if the operating stress is less than the lower limit or greater than
    the upper limit.  Limits and operating stresses may be integers or floats.

    >>> check_overstress(0.625, {'mild': [0.0, 0.9], 'harsh': [0.0, 0.75]})
    {'mild': [False, False], 'harsh': [False, False]}
    >>> check_overstress(-0.625, {'mild': [0.0, 0.9], 'harsh': [0.0, 0.75]})
    {'mild': [True, False], 'harsh': [True, False]}
    >>> check_overstress(0.825, {'mild': [0.0, 0.9], 'harsh': [0.0, 0.75]})
    {'mild': [False, False], 'harsh': [False, True]}
    >>> check_overstress(0.825, {'mild': [0, 0.9], 'harsh': [0, 0.75]})
    {'mild': [False, False], 'harsh': [False, True]}
    >>> check_overstress(1, {'mild': [0, 0.9], 'harsh': [0, 0.75]})
    {'mild': [False, True], 'harsh': [False, True]}

    Limits must be lists:
    >>> check_overstress(0.825, {'mild': 0.9, 'harsh': 0.75})
    Traceback (most recent call last):
        ...
    TypeError: 'float' object is not subscriptable

    And those lists must contain a lower and upper limit:
    check_overstress(0.825, {'mild': [0.9], 'harsh': [0.75]})
    Traceback (most recent call last):
        ...
    IndexError: list index out of range

    Limit values must not be strings:
    check_overstress(0.825, {'mild': [0.0, '0.9'], 'harsh': [0.0, 0.75]})
    Traceback (most recent call last):
        ...
    TypeError: '>' not supported between instances of 'float' and 'str'

    The programmer must ensure the operating stress and limits are provided in
    the proper format.  For example:

        op_stress = (operating current / rated current) with limits provided as
        decimals.
        op_stress = (operating temperature - maximum junction temperature) with
        limits provided as a delta T.

    :param op_stress: the level of the operating stress.
    :param limits: a dict containing the stress limits.  Key is the name
        of the environment (mild, harsh, protected, etc.) and the value is a
        list of [lower limit, upper limit].
    :return: _overstress; dict of indicators whether or not an overstress
        condition exists.  Key is the environment type (mild, harsh, protected,
        etc.) and the value is a list of booleans for each limit.
    :rtype: dict
    :raise: IndexError if a limit value has too few items in the list.
    :raise: TypeError if a limit value is not a list of numericals.
    """
    _overstress = {}

    for key in limits:
        _overstress[key] = [False, False]
        if op_stress < limits[key][0]:
            _overstress[key][0] = True
        if op_stress > limits[key][1]:
            _overstress[key][1] = True

    return _overstress



# Standard Library Imports
from typing import Dict, Union

# RAMSTK Package Imports
from ramstk.analyses import dormancy
from ramstk.analyses.milhdbk217f import milhdbk217f
from ramstk.analyses.statistics import exponential, lognormal, normal, weibull
from ramstk.db import RAMSTK_BASE as RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord as RAMSTKBaseRecord

class RAMSTKReliabilityRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    __defaults__: Dict[str, Union[float, int, str]]
    __tablename__: str
    __table_args__: Dict[str, bool]
    revision_id: int
    hardware_id: int
    add_adj_factor: float
    availability_logistics: float
    availability_mission: float
    avail_log_variance: float
    avail_mis_variance: float
    failure_distribution_id: int
    hazard_rate_active: float
    hazard_rate_dormant: float
    hazard_rate_logistics: float
    hazard_rate_method_id: int
    hazard_rate_mission: float
    hazard_rate_model: str
    hazard_rate_percent: float
    hazard_rate_software: float
    hazard_rate_specified: float
    hazard_rate_type_id: int
    hr_active_variance: float
    hr_dormant_variance: float
    hr_logistics_variance: float
    hr_mission_variance: float
    hr_specified_variance: float
    lambda_b: float
    location_parameter: float
    mtbf_logistics: float
    mtbf_mission: float
    mtbf_specified: float
    mtbf_logistics_variance: float
    mtbf_mission_variance: float
    mtbf_specified_variance: float
    mult_adj_factor: float
    quality_id: int
    reliability_goal: float
    reliability_goal_measure_id: int
    reliability_logistics: float
    reliability_mission: float
    reliability_log_variance: float
    reliability_miss_variance: float
    scale_parameter: float
    shape_parameter: float
    survival_analysis_id: int
    def get_attributes(self) -> Dict[str, Union[float, int, str]]: ...
    def do_calculate_hazard_rate_active(
        self,
        multiplier: float,
        attributes: Dict[str, Union[float, int, str]],
        time: float = 1.0,
    ) -> None: ...
    def do_calculate_hazard_rate_dormant(
        self,
        category_id: int,
        subcategory_id: int,
        env_active: int,
        env_dormant: int,
    ) -> None: ...
    def do_calculate_hazard_rate_logistics(self) -> None: ...
    def do_calculate_hazard_rate_mission(self, duty_cycle: float) -> None: ...
    def do_calculate_mtbf(self) -> None: ...
    def do_calculate_reliability(self, time: float) -> None: ...
    def do_predict_active_hazard_rate(
        self, attributes: Dict[str, Union[float, int, str]]
    ) -> None: ...



# Standard Library Imports
from typing import Any, Dict

# Third Party Imports
import pandas as pd

# RAMSTK Package Imports
from ramstk.db.base import BaseDatabase as BaseDatabase
from ramstk.exceptions import DataAccessError as DataAccessError
from ramstk.models import RAMSTKAllocationRecord as RAMSTKAllocationRecord
from ramstk.models import RAMSTKDesignElectricRecord as RAMSTKDesignElectricRecord
from ramstk.models import RAMSTKDesignMechanicRecord as RAMSTKDesignMechanicRecord
from ramstk.models import RAMSTKFunctionRecord as RAMSTKFunctionRecord
from ramstk.models import RAMSTKHardwareRecord as RAMSTKHardwareRecord
from ramstk.models import RAMSTKMilHdbk217FRecord as RAMSTKMilHdbk217FRecord
from ramstk.models import RAMSTKNSWCRecord as RAMSTKNSWCRecord
from ramstk.models import RAMSTKReliabilityRecord as RAMSTKReliabilityRecord
from ramstk.models import RAMSTKRequirementRecord as RAMSTKRequirementRecord
from ramstk.models import RAMSTKSimilarItemRecord as RAMSTKSimilarItemRecord
from ramstk.models import RAMSTKValidationRecord as RAMSTKValidationRecord

def _do_replace_nan(value: Any, default: Any) -> Any: ...
def _get_input_value(
    mapper: Dict[str, Any], df_row: pd.Series, field: str, default: Any
) -> Any: ...

class Import:
    _dic_field_map: Any
    _dao: Any
    _df_input_data: Any
    def __init__(self) -> None: ...
    def _do_connect(self, dao: BaseDatabase) -> None: ...
    def _do_import(self, module: str) -> None: ...
    def _do_insert_allocation(self, row: pd.Series) -> RAMSTKAllocationRecord: ...
    def _do_insert_design_electric(
        self, row: pd.Series
    ) -> RAMSTKDesignElectricRecord: ...
    def _do_insert_design_mechanic(
        self, row: pd.Series
    ) -> RAMSTKDesignMechanicRecord: ...
    def _do_insert_function(self, row: pd.Series) -> RAMSTKFunctionRecord: ...
    def _do_insert_hardware(self, row: pd.Series) -> RAMSTKHardwareRecord: ...
    def _do_insert_mil_hdbk_f(self, row: pd.Series) -> RAMSTKMilHdbk217FRecord: ...
    def _do_insert_nswc(self, row: pd.Series) -> RAMSTKNSWCRecord: ...
    def _do_insert_reliability(self, row: pd.Series) -> RAMSTKReliabilityRecord: ...
    def _do_insert_requirement(self, row: pd.Series) -> RAMSTKRequirementRecord: ...
    def _do_insert_similar_item(self, row: pd.Series) -> RAMSTKSimilarItemRecord: ...
    def _do_insert_validation(self, row: pd.Series) -> RAMSTKValidationRecord: ...
    def _do_map_to_field(
        self, module: str, import_field: str, format_field: str
    ) -> None: ...
    def _do_read_db_fields(self, module: str) -> None: ...
    def _do_read_file(self, file_type: str, file_name: str) -> None: ...



# Standard Library Imports
from typing import Any, Dict

# RAMSTK Package Imports
from ramstk.models import RAMSTKBaseView as RAMSTKBaseView

class RAMSTKFMEAView(RAMSTKBaseView):
    _root: int = ...
    _tag: str = ...
    _dic_load_functions: Any = ...
    _dic_trees: Any = ...
    _lst_modules: Any = ...
    def __init__(self, **kwargs: Dict[Any, Any]) -> None: ...
    def _do_load_modes(self) -> None: ...
    def _do_load_mechanisms(self, mode_id: int) -> None: ...
    def _do_load_causes(self, mechanism_id: int, parent_id: str) -> None: ...
    def _do_load_controls(self, cause_id: int, parent_id: str) -> None: ...
    def _do_load_actions(self, cause_id: int, parent_id: str) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.site_info.record.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Site Information Record Model."""

# Standard Library Imports
from datetime import date, timedelta

# Third Party Imports
from sqlalchemy import Column, Date, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKSiteInfoRecord(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent ramstk_site_info in the RAMSTK Common database."""

    __defaults__ = {
        "site_name": "",
        "product_key": "",
        "expire_on": date.today() + timedelta(30),
        "function_enabled": 0,
        "requirement_enabled": 0,
        "hardware_enabled": 0,
        "software_enabled": 0,
        "rcm_enabled": 0,
        "testing_enabled": 0,
        "incident_enabled": 0,
        "survival_enabled": 0,
        "vandv_enabled": 0,
        "hazard_enabled": 0,
        "stakeholder_enabled": 0,
        "allocation_enabled": 0,
        "similar_item_enabled": 0,
        "fmea_enabled": 0,
        "pof_enabled": 0,
        "rbd_enabled": 0,
        "fta_enabled": 0,
    }

    __tablename__ = "ramstk_site_info"
    __table_args__ = {"extend_existing": True}

    site_id = Column(
        "fld_site_id", Integer, primary_key=True, autoincrement=True, nullable=False
    )
    site_name = Column("fld_site_name", String(512), default=__defaults__["site_name"])
    product_key = Column(
        "fld_product_key", String(512), default=__defaults__["product_key"]
    )
    expire_on = Column("fld_expire_on", Date, default=__defaults__["expire_on"])
    function_enabled = Column(
        "fld_function_enabled", Integer, default=__defaults__["function_enabled"]
    )
    requirement_enabled = Column(
        "fld_requirement_enabled", Integer, default=__defaults__["requirement_enabled"]
    )
    hardware_enabled = Column(
        "fld_hardware_enabled", Integer, default=__defaults__["hardware_enabled"]
    )
    software_enabled = Column(
        "fld_software_enabled", Integer, default=__defaults__["software_enabled"]
    )
    rcm_enabled = Column(
        "fld_rcm_enabled", Integer, default=__defaults__["rcm_enabled"]
    )
    testing_enabled = Column(
        "fld_testing_enabled", Integer, default=__defaults__["testing_enabled"]
    )
    incident_enabled = Column(
        "fld_incident_enabled", Integer, default=__defaults__["incident_enabled"]
    )
    survival_enabled = Column(
        "fld_survival_enabled", Integer, default=__defaults__["survival_enabled"]
    )
    vandv_enabled = Column(
        "fld_vandv_enabled", Integer, default=__defaults__["vandv_enabled"]
    )
    hazard_enabled = Column(
        "fld_hazard_enabled", Integer, default=__defaults__["hazard_enabled"]
    )
    stakeholder_enabled = Column(
        "fld_stakeholder_enabled", Integer, default=__defaults__["stakeholder_enabled"]
    )
    allocation_enabled = Column(
        "fld_allocation_enabled", Integer, default=__defaults__["allocation_enabled"]
    )
    similar_item_enabled = Column(
        "fld_similar_item_enabled",
        Integer,
        default=__defaults__["similar_item_enabled"],
    )
    fmea_enabled = Column(
        "fld_fmea_enabled", Integer, default=__defaults__["fmea_enabled"]
    )
    pof_enabled = Column(
        "fld_pof_enabled", Integer, default=__defaults__["pof_enabled"]
    )
    rbd_enabled = Column(
        "fld_rbd_enabled", Integer, default=__defaults__["rbd_enabled"]
    )
    fta_enabled = Column(
        "fld_fta_enabled", Integer, default=__defaults__["fta_enabled"]
    )

    def get_attributes(self):
        """Retrieve current values of the RAMSTKSiteInfo data model attributes.

        :return: {site_id, product_key, expire_on, function_enabled,
                  requirement_enabled, vandv_enabled, fmea_enabled} pairs.
        :rtype: dict
        """
        _attributes = {
            "site_id": self.site_id,
            "site_name": self.site_name,
            "product_key": self.product_key,
            "expire_on": self.expire_on,
            "function_enabled": self.function_enabled,
            "requirement_enabled": self.requirement_enabled,
            "hardware_enabled": self.hardware_enabled,
            "software_enabled": self.software_enabled,
            "rcm_enabled": self.rcm_enabled,
            "testing_enabled": self.testing_enabled,
            "incident_enabled": self.incident_enabled,
            "survival_enabled": self.survival_enabled,
            "vandv_enabled": self.vandv_enabled,
            "hazard_enabled": self.hazard_enabled,
            "stakeholder_enabled": self.stakeholder_enabled,
            "allocation_enabled": self.allocation_enabled,
            "similar_item_enabled": self.similar_item_enabled,
            "fmea_enabled": self.fmea_enabled,
            "pof_enabled": self.pof_enabled,
            "rbd_enabled": self.rbd_enabled,
            "fta_enabled": self.fta_enabled,
        }

        return _attributes



# -*- coding: utf-8 -*-
#
#       ramstk.models.commondb.RAMSTKLoadHistory.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2007 - 2017 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""RAMSTKLoadHistory Table."""

# Third Party Imports
from sqlalchemy import Column, Integer, String

# RAMSTK Package Imports
from ramstk.db import RAMSTK_BASE
from ramstk.models import RAMSTKBaseRecord


class RAMSTKLoadHistory(RAMSTK_BASE, RAMSTKBaseRecord):
    """Class to represent the table ramstk_load_history."""

    __defaults__ = {"description": "Load History Description"}
    __tablename__ = "ramstk_load_history"
    __table_args__ = {"extend_existing": True}

    history_id = Column(
        "fld_history_id", Integer, primary_key=True, autoincrement=True, nullable=False
    )
    description = Column(
        "fld_description", String(512), default=__defaults__["description"]
    )

    def get_attributes(self):
        """Retrieve current values of RAMSTKLoadHistory data model attributes.

        :return: {load_history_id, description} pairs
        :rtype: dict
        """
        _values = {
            "history_id": self.history_id,
            "description": self.description,
        }

        return _values



# Standard Library Imports
from typing import Any, Dict, List

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKCheckButton as RAMSTKCheckButton
from ramstk.views.gtk3.widgets import RAMSTKDialog as RAMSTKDialog
from ramstk.views.gtk3.widgets import RAMSTKEntry as RAMSTKEntry
from ramstk.views.gtk3.widgets import RAMSTKLabel as RAMSTKLabel
from ramstk.views.gtk3.widgets import RAMSTKTreeView as RAMSTKTreeView

class EditFunction(RAMSTKDialog):
    chkApplyAll: Any = ...
    txtFunction1: Any = ...
    txtFunction2: Any = ...
    txtFunction3: Any = ...
    txtFunction4: Any = ...
    txtFunction5: Any = ...
    def __init__(self, treeview: RAMSTKTreeView, **kwargs: Dict[str, Any]) -> None: ...
    def __make_ui(self) -> None: ...
    def do_set_functions(self, treeview: RAMSTKTreeView) -> List[str]: ...
    def _cancel(self, __button: Gtk.Button) -> None: ...
    def _do_load_functions(self, treeview: RAMSTKTreeView) -> None: ...
    def _on_set_function(self, hardware_id: int) -> None: ...



# Standard Library Imports
from typing import Tuple

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKSiteConfiguration as RAMSTKSiteConfiguration
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.db import BaseDatabase as BaseDatabase
from ramstk.db.common import do_load_variables as do_load_variables
from ramstk.exim import Export as Export
from ramstk.exim import Import as Import
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.models import RAMSTKActionTable as RAMSTKActionTable
from ramstk.models import RAMSTKAllocationTable as RAMSTKAllocationTable
from ramstk.models import RAMSTKCauseTable as RAMSTKCauseTable
from ramstk.models import RAMSTKControlTable as RAMSTKControlTable
from ramstk.models import RAMSTKDesignElectricTable as RAMSTKDesignElectricTable
from ramstk.models import RAMSTKDesignMechanicTable as RAMSTKDesignMechanicTable
from ramstk.models import RAMSTKEnvironmentTable as RAMSTKEnvironmentTable
from ramstk.models import RAMSTKFailureDefinitionTable as RAMSTKFailureDefinitionTable
from ramstk.models import RAMSTKFMEAView as RAMSTKFMEAView
from ramstk.models import RAMSTKFunctionTable as RAMSTKFunctionTable
from ramstk.models import RAMSTKHardwareBoMView as RAMSTKHardwareBoMView
from ramstk.models import RAMSTKHardwareTable as RAMSTKHardwareTable
from ramstk.models import RAMSTKHazardTable as RAMSTKHazardTable
from ramstk.models import RAMSTKMechanismTable as RAMSTKMechanismTable
from ramstk.models import RAMSTKMILHDBK217FTable as RAMSTKMILHDBK217FTable
from ramstk.models import RAMSTKMissionPhaseTable as RAMSTKMissionPhaseTable
from ramstk.models import RAMSTKMissionTable as RAMSTKMissionTable
from ramstk.models import RAMSTKModeTable as RAMSTKModeTable
from ramstk.models import RAMSTKNSWCTable as RAMSTKNSWCTable
from ramstk.models import RAMSTKOpLoadTable as RAMSTKOpLoadTable
from ramstk.models import RAMSTKOpStressTable as RAMSTKOpStressTable
from ramstk.models import RAMSTKPoFView as RAMSTKPoFView
from ramstk.models import RAMSTKProgramDB as RAMSTKProgramDB
from ramstk.models import RAMSTKProgramInfoTable as RAMSTKProgramInfoTable
from ramstk.models import RAMSTKProgramStatusTable as RAMSTKProgramStatusTable
from ramstk.models import RAMSTKReliabilityTable as RAMSTKReliabilityTable
from ramstk.models import RAMSTKRequirementTable as RAMSTKRequirementTable
from ramstk.models import RAMSTKRevisionTable as RAMSTKRevisionTable
from ramstk.models import RAMSTKSimilarItemTable as RAMSTKSimilarItemTable
from ramstk.models import RAMSTKSiteInfoTable as RAMSTKSiteInfoTable
from ramstk.models import RAMSTKStakeholderTable as RAMSTKStakeholderTable
from ramstk.models import RAMSTKTestMethodTable as RAMSTKTestMethodTable
from ramstk.models import RAMSTKUsageProfileView as RAMSTKUsageProfileView
from ramstk.models import RAMSTKValidationTable as RAMSTKValidationTable
from ramstk.utilities import file_exists as file_exists
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import RAMSTKDesktop as RAMSTKDesktop
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.widgets import RAMSTKDatabaseSelect as RAMSTKDatabaseSelect

def do_connect_to_site_db(conn_info) -> BaseDatabase: ...
def do_first_run(configuration: RAMSTKSiteConfiguration) -> None: ...
def do_initialize_databases(
    configuration: RAMSTKUserConfiguration, site_db: BaseDatabase
) -> RAMSTKProgramDB: ...
def do_initialize_loggers(log_file: str, log_level: str) -> RAMSTKLogManager: ...
def do_read_site_configuration() -> RAMSTKSiteConfiguration: ...
def do_read_user_configuration() -> Tuple[
    RAMSTKUserConfiguration, RAMSTKLogManager
]: ...
def the_one_ring() -> None: ...



# Standard Library Imports
from typing import Any

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.views.gtk3.function import FunctionWorkView as FunctionWorkView
from ramstk.views.gtk3.revision import RevisionWorkView as RevisionWorkView
from ramstk.views.gtk3.widgets import RAMSTKBaseBook as RAMSTKBaseBook
from ramstk.views.gtk3.widgets import RAMSTKBaseView as RAMSTKBaseView

class RAMSTKWorkBook(RAMSTKBaseBook):
    dic_work_views: Any
    def __init__(
        self, configuration: RAMSTKUserConfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def _on_module_change(self, module: str = ...) -> None: ...



# pylint: disable=unused-import
# -*- coding: utf-8 -*-
#
#       ramstk.analyses.statistics.__init__.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK statistical analyses package."""

# RAMSTK Local Imports
from . import exponential, lognormal, normal, weibull
from .bounds import do_calculate_beta_bounds, do_calculate_fisher_information



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.pof.panel.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 PoF Panels."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
import treelib
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import GdkPixbuf, Gtk, _
from ramstk.views.gtk3.widgets import RAMSTKTreePanel


class PoFTreePanel(RAMSTKTreePanel):
    """Panel to display Physics if Failure analysis worksheet."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _select_msg = "succeed_retrieve_modes"
    _tag = "pof"
    _title = _("Physics of Failure (PoF) Analysis")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the PoF analysis worksheet."""
        super().__init__()

        # Initialize private dictionary instance attributes.
        self.tvwTreeView.dic_row_loader = {
            "mode": self.__do_load_mode,
            "mechanism": self.__do_load_mechanism,
            "opload": self.__do_load_opload,
            "opstress": self.__do_load_opstress,
            "method": self.__do_load_test_method,
        }
        self._dic_visible_mask: Dict[str, List[str]] = {
            "mode": [
                "True",
                "True",
                "True",
                "True",
                "True",
                "False",
                "False",
                "False",
                "False",
                "False",
                "False",
                "False",
            ],
            "mechanism": [
                "True",
                "True",
                "True",
                "True",
                "True",
                "False",
                "False",
                "False",
                "False",
                "False",
                "False",
                "False",
            ],
            "opload": [
                "True",
                "True",
                "True",
                "True",
                "True",
                "False",
                "False",
                "False",
                "False",
                "True",
                "False",
                "False",
            ],
            "opstress": [
                "True",
                "True",
                "True",
                "True",
                "True",
                "False",
                "True",
                "True",
                "False",
                "False",
                "True",
                "False",
            ],
            "testmethod": [
                "True",
                "True",
                "True",
                "True",
                "True",
                "False",
                "False",
                "False",
                "True",
                "False",
                "True",
                "False",
            ],
        }

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.
        self._on_edit_message: str = f"wvw_editing_{self._tag}"

        # Initialize public dictionary instance attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "mode_id": [
                0,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Mode ID"),
                "gint",
            ],
            "mechanism_id": [
                1,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Mechanism ID"),
                "gint",
            ],
            "load_id": [
                2,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Load ID"),
                "gint",
            ],
            "stress_id": [
                3,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Stress ID"),
                "gint",
            ],
            "test_id": [
                4,
                Gtk.CellRendererText(),
                "edited",
                None,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": False,
                    "fg_color": "#000000",
                    "visible": False,
                },
                _("Test ID"),
                "gint",
            ],
            "description": [
                5,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Description"),
                "gchararray",
            ],
            "effect_end": [
                6,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("End Effect"),
                "gchararray",
            ],
            "severity_class": [
                7,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Severity"),
                "gchararray",
            ],
            "mode_probability": [
                8,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0.0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Mode Probability"),
                "gfloat",
            ],
            "damage_model": [
                9,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_toggled,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Damage Model"),
                "gchararray",
            ],
            "measurable_parameter": [
                10,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Measurable Parameter"),
                "gchararray",
            ],
            "load_history": [
                11,
                Gtk.CellRendererCombo(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Load History Method"),
                "gchararray",
            ],
            "boundary_conditions": [
                12,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Boundary Conditions"),
                "gchararray",
            ],
            "priority_id": [
                13,
                Gtk.CellRendererSpin(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                0,
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Priority"),
                "gint",
            ],
            "remarks": [
                14,
                Gtk.CellRendererText(),
                "edited",
                super().on_cell_edit,
                self._on_edit_message,
                "",
                {
                    "bg_color": "#FFFFFF",
                    "editable": True,
                    "fg_color": "#000000",
                    "visible": True,
                },
                _("Remarks"),
                "gchararray",
            ],
        }
        self.dic_icons: Dict[str, str] = {}

        # Initialize public list instance attributes.
        self.lst_damage_models: List[str] = []
        self.lst_load_history: List[str] = []
        self.lst_measurable_parameters: List[str] = []

        # Initialize public scalar instance attributes.

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        self.tvwTreeView.set_tooltip_text(
            _(
                "Displays the Physics of Failure (PoF) Analysis for the currently "
                "selected hardware item."
            )
        )

        # Subscribe to PyPubSub messages.
        pub.subscribe(super().do_load_panel, "succeed_delete_test_method")
        pub.subscribe(super().do_load_panel, "succeed_delete_opstress")
        pub.subscribe(super().do_load_panel, "succeed_delete_opload")
        pub.subscribe(super().do_load_panel, "succeed_delete_mechanism")
        pub.subscribe(super().do_load_panel, "succeed_delete_mode")

    def do_load_comboboxes(self) -> None:
        """Load the RAMSTKComboBox() widgets.

        :return: None
        :rtype: None
        """
        self.__do_load_damage_models()
        self.__do_load_measureable_parameters()
        self.__do_load_load_history()

        # Set the priority Gtk.CellRendererSpin()'s adjustment limits and
        # step increments.
        _cell = self.tvwTreeView.get_column(
            self.tvwTreeView.position["priority_id"]
        ).get_cells()[0]
        _adjustment = _cell.get_property("adjustment")
        _adjustment.configure(5, 1, 5, -1, 0, 0)

    def _on_row_change(self, selection: Gtk.TreeSelection) -> None:
        """Handle events for the PoF Work View RAMSTKTreeView().

        This method is called whenever a RAMSTKTreeView() row is activated.

        :param selection: the TreeSelection() of the currently
            selected row in the PoF RAMSTKTreeView().
        :return: None
        """
        _model, _row = selection.get_selected()

        if _row is not None:
            if _model.get_value(_row, 0) == 0:
                _level = "mode"
            elif _model.get_value(_row, 1) == 0:
                _level = "mechanism"
            elif _model.get_value(_row, 2) == 0:
                _level = "load"
            elif _model.get_value(_row, 3) == 0:
                _level = "stress"
            else:
                _level = "test"

            self.tvwTreeView.visible = self._dic_visible_mask[_level]
            self.tvwTreeView.do_set_visible_columns()

    def __do_load_damage_models(self) -> None:
        """Load the RAMSTKTreeView() damage model CellRendererCombo().

        :return: None
        """
        self.tvwTreeView.do_load_combo_cell(
            self.tvwTreeView.position["damage_model"], self.lst_damage_models
        )

    def __do_load_load_history(self) -> None:
        """Load the operating load history CellRendererCombo().

        :return: None
        :rtype: None
        """
        self.tvwTreeView.do_load_combo_cell(
            self.tvwTreeView.position["load_history"], self.lst_load_history
        )

    def __do_load_measureable_parameters(self) -> None:
        """Load the measureable parameters CellRendererCombo().

        :return: None
        """
        self.tvwTreeView.do_load_combo_cell(
            self.tvwTreeView.position["measurable_parameter"],
            self.lst_measurable_parameters,
        )

    def __do_load_mechanism(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter:
        """Load a failure mechanism record into the RAMSTKTreeView().

        :param node: the treelib Node() with the mechanism data to load.
        :param row: the parent row of the mechanism to load into the FMEA form.
        :return: _new_row; the row that was just populated with mechanism data.
        """
        _new_row = None

        [[__, _entity]] = node.data.items()  # pylint: disable=unused-variable

        _model = self.tvwTreeView.get_model()

        # noinspection PyArgumentList
        _icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
            self.dic_icons["mechanism"], 22, 22
        )

        _attributes = [
            _entity.mode_id,
            _entity.mechanism_id,
            0,
            0,
            0,
            _entity.description,
            "",
            "",
            0.0,
            "",
            "",
            "",
            "",
            0,
            "",
            _icon,
        ]

        try:
            _new_row = _model.append(row, _attributes)
        except (AttributeError, TypeError, ValueError):
            _new_row = None
            _message = _(
                "An error occurred when loading failure mechanism {0:s} in "
                "the "
                "physics of failure analysis.  This might indicate it was "
                "missing it's data package, some of the data in the package "
                "was missing, or some of the data was the wrong type.  Row "
                "data was: {1}"
            ).format(str(node.identifier), _attributes)
            pub.sendMessage(
                "do_log_warning_msg", logger_name="WARNING", message=_message
            )

        return _new_row

    def __do_load_mode(self, node: treelib.Node, row: Gtk.TreeIter) -> Gtk.TreeIter:
        """Load a failure mode record into the RAMSTKTreeView().

        :param node: the treelib Node() with the mode data to load.
        :param row: the parent row of the mode to load into the FMEA form.
        :return: _new_row; the row that was just populated with mode data.
        """
        _new_row = None

        [[__, _entity]] = node.data.items()  # pylint: disable=unused-variable

        _model = self.tvwTreeView.get_model()

        # noinspection PyArgumentList
        _icon = GdkPixbuf.Pixbuf.new_from_file_at_size(self.dic_icons["mode"], 22, 22)

        _attributes = [
            _entity.mode_id,
            0,
            0,
            0,
            0,
            _entity.description,
            _entity.effect_end,
            _entity.severity_class,
            _entity.mode_ratio,
            "",
            "",
            "",
            "",
            0,
            "",
            _icon,
        ]

        try:
            _new_row = _model.append(row, _attributes)
        except (AttributeError, TypeError, ValueError):
            _new_row = None
            _message = _(
                "An error occurred when loading failure mode {0:s} in the "
                "physics of failure analysis.  This might indicate it was "
                "missing it's data package, some of the data in the package "
                "was missing, or some of the data was the wrong type.  Row "
                "data was: {1}"
            ).format(str(node.identifier), _attributes)
            pub.sendMessage(
                "do_log_warning_msg", logger_name="WARNING", message=_message
            )

        return _new_row

    def __do_load_opload(self, node: treelib.Node, row: Gtk.TreeIter) -> Gtk.TreeIter:
        """Load a failure mechanism record into the RAMSTKTreeView().

        :param node: the treelib Node() with the mechanism data to load.
        :param row: the parent row of the mechanism to load into the FMEA form.
        :return: _new_row; the row that was just populated with mechanism data.
        """
        _new_row = None

        [[__, _entity]] = node.data.items()  # pylint: disable=unused-variable

        _model = self.tvwTreeView.get_model()

        # noinspection PyArgumentList
        _icon = GdkPixbuf.Pixbuf.new_from_file_at_size(self.dic_icons["opload"], 22, 22)

        _damage_model = self.dic_damage_models[_entity.damage_model]

        _attributes = [
            _entity.mode_id,
            _entity.mechanism_id,
            _entity.load_id,
            0,
            0,
            _entity.description,
            "",
            "",
            0.0,
            _damage_model,
            "",
            "",
            "",
            _entity.priority_id,
            "",
            _icon,
        ]

        try:
            _new_row = _model.append(row, _attributes)
        except (AttributeError, TypeError, ValueError):
            _new_row = None
            _message = _(
                "An error occurred when loading operating load {0:s} in the "
                "physics of failure analysis.  This might indicate it was "
                "missing it's data package, some of the data in the package "
                "was missing, or some of the data was the wrong type.  Row "
                "data was: {1}"
            ).format(str(node.identifier), _attributes)
            pub.sendMessage(
                "do_log_warning_msg", logger_name="WARNING", message=_message
            )

        return _new_row

    def __do_load_opstress(self, node: treelib.Node, row: Gtk.TreeIter) -> Gtk.TreeIter:
        """Load a failure mechanism record into the RAMSTKTreeView().

        :param node: the treelib Node() with the mechanism data to load.
        :param row: the parent row of the mechanism to load into the FMEA form.
        :return: _new_row; the row that was just populated with mechanism data.
        """
        _new_row = None

        [[__, _entity]] = node.data.items()  # pylint: disable=unused-variable

        _model = self.tvwTreeView.get_model()

        # noinspection PyArgumentList
        _icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
            self.dic_icons["opstress"], 22, 22
        )

        _load_history = self.dic_load_history[_entity.load_history]
        _measurable_parameter = self.dic_measurable_parameters[
            _entity.measurable_parameter
        ]

        _attributes = [
            _entity.mode_id,
            _entity.mechanism_id,
            _entity.load_id,
            _entity.stress_id,
            0,
            _entity.description,
            "",
            "",
            0.0,
            "",
            _measurable_parameter,
            _load_history,
            "",
            0,
            _entity.remarks,
            _icon,
        ]

        try:
            _new_row = _model.append(row, _attributes)
        except (AttributeError, TypeError, ValueError):
            _new_row = None
            _message = _(
                "An error occurred when loading operating stress {0:s} in the "
                "physics of failure analysis.  This might indicate it was "
                "missing it's data package, some of the data in the package "
                "was missing, or some of the data was the wrong type.  Row "
                "data was: {1}"
            ).format(str(node.identifier), _attributes)
            pub.sendMessage(
                "do_log_warning_msg", logger_name="WARNING", message=_message
            )

        return _new_row

    def __do_load_test_method(
        self, node: treelib.Node, row: Gtk.TreeIter
    ) -> Gtk.TreeIter:
        """Load a failure mechanism record into the RAMSTKTreeView().

        :param node: the treelib Node() with the mechanism data to load.
        :param row: the parent row of the mechanism to load into the FMEA form.
        :return: _new_row; the row that was just populated with mechanism data.
        """
        _new_row = None

        [[__, _entity]] = node.data.items()  # pylint: disable=unused-variable

        _model = self.tvwTreeView.get_model()

        # noinspection PyArgumentList
        _icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
            self.dic_icons["testmethod"], 22, 22
        )

        _attributes = [
            _entity.mode_id,
            _entity.mechanism_id,
            _entity.load_id,
            _entity.stress_id,
            _entity.test_id,
            _entity.description,
            "",
            "",
            0.0,
            "",
            "",
            "",
            _entity.boundary_conditions,
            0,
            _entity.remarks,
            _icon,
        ]

        try:
            _new_row = _model.append(row, _attributes)
        except (AttributeError, TypeError, ValueError):
            _new_row = None
            _message = _(
                "An error occurred when loading test method {0:s} in the "
                "physics of failure analysis.  This might indicate it was "
                "missing it's data package, some of the data in the package "
                "was missing, or some of the data was the wrong type.  Row "
                "data was: {1}"
            ).format(str(node.identifier), _attributes)
            pub.sendMessage(
                "do_log_warning_msg", logger_name="WARNING", message=_message
            )

        return _new_row



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.components.inductor.py is part of the RAMSTK
#       Project.
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""Inductive Device Input Panel."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKComboBox, RAMSTKEntry, RAMSTKFixedPanel


class InductorDesignElectricInputPanel(RAMSTKFixedPanel):
    """Displays Inductor assessment input attribute data.

    The Inductor assessment input view displays all the assessment inputs for
    the selected inductor.  This includes, currently, inputs for
    MIL-HDBK-217FN2 parts count and part stress analysis.  The attributes of an
    Inductor assessment input view are:

    :cvar _dic_insulation: dictionary of insulation classes.  Key is
        inductor subcategory ID; values are lists of insulation classes.
    :cvar dict _dic_specifications: dictionary of inductor MIL-SPECs.  Key is
        inductor subcategory ID; values are lists of specifications.

    :ivar list _lst_labels: list of label text to display for the capacitor
        MIL-HDBK-217 input parameters.
    :ivar _lst_widgets: the list of widgets to display in the panel.  These
        are listed in the order they should appear on the panel.

    :ivar cmbInsulation: select and display the insulation class of the
        inductor.
    :ivar cmbSpecification: select and display the governing specification for
        the inductor.
    :ivar cmbConstruction: select and display the method of construction of the
        inductor.
    :ivar cmbFamily: select and display the family of the transformer.
    :ivar txtArea: enter and display the heat dissipating area of the inductor.
    :ivar txtWeight: enter and display the weight of the inductor.
    """

    # Define private dict class attributes.
    _dic_insulation: Dict[int, List[List[str]]] = {
        1: [
            [_("Insulation Class A")],
            [_("Insulation Class B")],
            [_("Insulation Class C")],
            [_("Insulation Class O")],
            [_("Insulation Class Q")],
            [_("Insulation Class R")],
            [_("Insulation Class S")],
            [_("Insulation Class T")],
            [_("Insulation Class U")],
            [_("Insulation Class V")],
        ],
        2: [
            [_("Insulation Class A")],
            [_("Insulation Class B")],
            [_("Insulation Class C")],
            [_("Insulation Class F")],
            [_("Insulation Class O")],
        ],
    }
    _dic_quality: Dict[int, List[List[str]]] = {
        1: [["MIL-SPEC"], [_("Lower")]],
        2: [["S"], ["R"], ["P"], ["M"], ["MIL-C-15305"], [_("Lower")]],
    }
    _dic_specifications: Dict[int, List[List[str]]] = {
        1: [["MIL-T-27"], ["MIL-T-21038"], ["MIL-T-55631"]],
        2: [["MIL-T-15305"], ["MIL-T-39010"]],
    }

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "selected_hardware"
    _tag: str = "design_electric"
    _title: str = _("Inductive Device Design Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Inductor assessment input view."""
        super().__init__()

        # Initialize widgets.
        self.cmbConstruction: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbFamily: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbInsulation: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbQuality: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbSpecification: RAMSTKComboBox = RAMSTKComboBox()
        self.txtArea: RAMSTKEntry = RAMSTKEntry()
        self.txtWeight: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.
        self._hazard_rate_method_id: int = 0
        self._quality_id: int = 0

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "quality_id": [
                32,
                self.cmbQuality,
                "changed",
                super().on_changed_combo,
                "wvw_editing_reliability",
                0,
                {
                    "tooltip": _("The quality level of the inductive device."),
                },
                _("Quality Level:"),
                "gint",
            ],
            "specification_id": [
                36,
                self.cmbSpecification,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _(
                        "The governing specification for the inductive device."
                    ),
                },
                _("Specification:"),
                "gint",
            ],
            "insulation_id": [
                19,
                self.cmbInsulation,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The insulation class of the inductive device."),
                },
                _("Insulation Class:"),
                "gint",
            ],
            "area": [
                3,
                self.txtArea,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The case radiating surface (in square inches) of the "
                        "inductive device."
                    ),
                },
                _("Area:"),
                "gfloat",
            ],
            "weight": [
                54,
                self.txtWeight,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The transformer weight (in lbf)."),
                },
                _("Weight:"),
                "gfloat",
            ],
            "family_id": [
                15,
                self.cmbFamily,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The application family of the transformer."),
                },
                _("Family:"),
                "gint",
            ],
            "construction_id": [
                6,
                self.cmbConstruction,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0,
                {
                    "tooltip": _("The method of construction of the coil."),
                },
                _("Construction:"),
                "gint",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.category_id: int = 0
        self.subcategory_id: int = 0

        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self.do_load_comboboxes,
            "changed_subcategory",
        )
        pub.subscribe(
            self._do_set_reliability_attributes,
            "succeed_get_reliability_attributes",
        )

    def do_load_comboboxes(self, subcategory_id: int) -> None:
        """Load the inductive device assessment input RAMSTKComboBox().

        :param subcategory_id: the subcategory ID of the selected inductive device.
        :return: None
        :rtype: None
        """
        self.subcategory_id = subcategory_id

        self.__do_load_family_combobox()
        self.__do_load_insulation_combobox()
        self.__do_load_quality_combobox()
        self.__do_load_specification_combobox()

        self.cmbConstruction.do_load_combo(
            [[_("Fixed")], [_("Variable")]], signal="changed"
        )

    def _do_load_panel(self, attributes: Dict[str, Any]) -> None:
        """Load the Inductor assessment input widgets.

        :param attributes: the attributes dictionary for the selected
        Inductor.
        :return: None
        :rtype: None
        """
        super().do_load_common(attributes)

        self.cmbFamily.do_update(attributes["family_id"], signal="changed")

        if self._hazard_rate_method_id == 2:
            self.cmbSpecification.do_update(
                attributes["specification_id"], signal="changed"
            )
            self.cmbInsulation.do_update(attributes["insulation_id"], signal="changed")
            self.cmbConstruction.do_update(
                attributes["construction_id"], signal="changed"
            )
            self.txtArea.do_update(
                str(self.fmt.format(attributes["area"])), signal="changed"
            )
            self.txtWeight.do_update(
                str(self.fmt.format(attributes["weight"])), signal="changed"
            )  # noqa

    def _do_set_reliability_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._hazard_rate_method_id = attributes["hazard_rate_method_id"]
            self._quality_id = attributes["quality_id"]

            self.cmbQuality.set_sensitive(True)
            self.cmbQuality.do_update(
                self._quality_id,
                signal="changed",
            )

            pub.sendMessage(
                f"request_get_{self._tag}_attributes",
                node_id=self._record_id,
            )

    def _do_set_sensitive(self, attributes: Dict[str, Any]) -> None:
        """Set widget sensitivity as needed for the selected inductor.

        :return: None
        :rtype: None
        """
        self.cmbSpecification.set_sensitive(False)
        self.cmbInsulation.set_sensitive(False)
        self.cmbFamily.set_sensitive(False)
        self.cmbConstruction.set_sensitive(False)
        self.txtArea.set_sensitive(False)
        self.txtWeight.set_sensitive(False)

        if self._hazard_rate_method_id == 1:
            self.cmbFamily.set_sensitive(True)
            self.cmbFamily.do_update(
                attributes["family_id"],
                signal="changed",
            )
        else:
            self.cmbSpecification.set_sensitive(True)
            self.cmbSpecification.do_update(
                attributes["specification_id"],
                signal="changed",
            )
            self.cmbInsulation.set_sensitive(True)
            self.cmbInsulation.do_update(
                attributes["insulation_id"],
                signal="changed",
            )
            self.txtArea.set_sensitive(True)
            self.txtArea.do_update(
                attributes["area"],
                signal="changed",
            )
            self.txtWeight.set_sensitive(True)
            self.txtWeight.do_update(
                attributes["weight"],
                signal="changed",
            )

            if self.subcategory_id == 1:
                self.cmbFamily.set_sensitive(True)
                self.cmbFamily.do_update(
                    attributes["family_id"],
                    signal="changed",
                )

            if self.subcategory_id == 2:
                self.cmbConstruction.set_sensitive(True)
                self.cmbConstruction.do_update(
                    attributes["construction_id"],
                    signal="changed",
                )

    def __do_load_family_combobox(self) -> None:
        """Load the family RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 1:
            if self.subcategory_id == 1:
                _data = [
                    [_("Low Power Pulse Transformer")],
                    [_("Audio Transformer")],
                    [_("High Power Pulse and Power Transformer, Filter")],
                    [_("RF Transformer")],
                ]
            else:
                _data = [[_("RF Coils, Fixed or Molded")], [_("RF Coils, Variable")]]
        else:
            _data = [
                [_("Pulse Transformer")],
                [_("Audio Transformer")],
                [_("Power Transformer or Filter")],
                [_("RF Transformer")],
            ]
        self.cmbFamily.do_load_combo(_data, signal="changed")

    def __do_load_insulation_combobox(self) -> None:
        """Load the insulation RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        try:
            _data = self._dic_insulation[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbInsulation.do_load_combo(_data, signal="changed")

    def __do_load_quality_combobox(self) -> None:
        """Load the quality RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        if self._hazard_rate_method_id == 1:
            _data = [[_("Established Reliability")], ["MIL-SPEC"], [_("Lower")]]
        else:
            try:
                _data = self._dic_quality[self.subcategory_id]
            except KeyError:
                _data = []
        self.cmbQuality.do_load_combo(_data, signal="changed")

    def __do_load_specification_combobox(self) -> None:
        """Load the specification RAMSTKComboBox().

        :return: None
        :rtype: None
        """
        try:
            _data = self._dic_specifications[self.subcategory_id]
        except KeyError:
            _data = []
        self.cmbSpecification.do_load_combo(_data, signal="changed")



# -*- coding: utf-8 -*-
#
#       ramstk.models.programdb.database.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RAMSTK program Database model."""

# Standard Library Imports
from typing import Dict

# Third Party Imports
from pubsub import pub

# noinspection PyPackageRequirements
from sqlalchemy.exc import (  # type: ignore
    ArgumentError,
    NoSuchModuleError,
    OperationalError,
)

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKUserConfiguration
from ramstk.db import BaseDatabase, do_create_program_db
from ramstk.exceptions import DataAccessError


class RAMSTKProgramDB:
    """The RAMSTK program manager class.

    The RAMSTK program manager is responsible for managing all the analysis,
    data, and matrix managers associated with the RAMSTK program database that
    is currently open.  The attributes of a RAMSTK program manager are:

    :ivar dict dic_managers: a dict containing the instances of all the
        analysis, data, and matrix managers associated with the currently
        active run of RAMSTK.  The first key is the workstream module name
        which has a dict as a value.  The key of this second dict is the type
        of manager (analysis, data, matrix) and the value will be the instance
        of the applicable manager.
    :ivar program_dao: the BaseDatabase() object that will connect to the
        RAMSTK program database.
    :type program_dao: :class:`ramstk.db.base.BaseDatabase`
    """

    def __init__(self) -> None:
        """Initialize an instance of the RAMSTK program manager."""
        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.
        self.dic_tables: Dict[str, object] = {
            "action": object,
            "allocation": object,
            "cause": object,
            "control": object,
            "design_electric": object,
            "design_mechanic": object,
            "environment": object,
            "failure_definition": object,
            "function": object,
            "hardware": object,
            "hazards": object,
            "mechanism": object,
            "milhdbk217f": object,
            "mission": object,
            "mission_phase": object,
            "mode": object,
            "nswc": object,
            "opload": object,
            "opstress": object,
            "program_info": object,
            "program_status": object,
            "reliability": object,
            "requirement": object,
            "revision": object,
            "similar_item": object,
            "stakeholder": object,
            "test_method": object,
            "validation": object,
            "options": object,
            "export": object,
            "import": object,
        }
        self.dic_views: Dict[str, object] = {
            "fmea": object,
            "hardwarebom": object,
            "pof": object,
            "usage_profile": object,
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.user_configuration: RAMSTKUserConfiguration = RAMSTKUserConfiguration()
        self.program_dao: BaseDatabase = BaseDatabase()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self.do_create_program, "request_create_program")
        pub.subscribe(self.do_open_program, "request_open_program")
        pub.subscribe(self.do_open_program, "succeed_create_program_database")
        pub.subscribe(self.do_close_program, "request_close_program")
        pub.subscribe(self.do_save_program, "request_update_program")

    def do_create_program(
        self, program_db: BaseDatabase, database: Dict[str, str]
    ) -> None:
        """Create a new RAMSTK Program database.

        :param program_db: the BaseDatabase() that is to be used to create and connect
            to the new RAMSTK program database.
        :param database: a dict containing the database connection arguments.
        :return: None
        :rtype: None
        """
        with open(
            self.user_configuration.RAMSTK_CONF_DIR
            + f'/{database["dialect"]}_program_db.sql',
            "r",
        ) as _sql_file:
            self.program_dao = program_db
            do_create_program_db(database, _sql_file)
            pub.sendMessage(
                "succeed_create_program_database",
                program_db=self.program_dao,
                database=database,
            )

    def do_open_program(
        self, program_db: BaseDatabase, database: Dict[str, str]
    ) -> None:
        """Open an RAMSTK Program database for analyses.

        :param program_db: the BaseDatabase() that is to be used to create and
            connect to the new RAMSTK program database.
        :param database: a dictionary of parameters to pass to the DAO.
        :return: None
        :rtype: None
        """
        self.program_dao = program_db

        try:
            self.program_dao.do_connect(database)
            pub.sendMessage("succeed_connect_program_database", dao=self.program_dao)
            pub.sendMessage(
                "request_retrieve_revisions", attributes={"revision_id": None}
            )
        except NoSuchModuleError:
            _error_msg = (
                "RAMSTK does not currently support database dialect "
                "{0:s}.".format(database["dialect"])
            )
            pub.sendMessage("fail_connect_program_database", error_message=_error_msg)
        except ArgumentError:
            _error_msg = (
                "The database URL {0:s} did not conform to the RFC 1738 "
                "standard and could not be opened.".format(database["database"])
            )
            pub.sendMessage("fail_connect_program_database", error_message=_error_msg)
        except OperationalError:
            _error_msg = "The database {0:s} does not exist.".format(
                database["database"]
            )
            pub.sendMessage("fail_connect_program_database", error_message=_error_msg)
        except DataAccessError as _error:
            pub.sendMessage("fail_connect_program_database", error_message=_error.msg)

    def do_close_program(self) -> None:
        """Close the open RAMSTK Program database.

        :return: None
        :rtype: None
        :raises: AttributeError if a database isn't open.
        """
        try:
            self.program_dao.do_disconnect()
            pub.sendMessage("succeed_disconnect_program_database")
        except AttributeError:
            _error_msg = "Not currently connected to a database.  Nothing to close."
            pub.sendMessage(
                "fail_disconnect_program_database", error_message=_error_msg
            )

    @staticmethod
    def do_save_program() -> None:
        """Save the open RAMSTK Program database.

        :return: None
        :rtype: None
        """
        pub.sendMessage("request_update_all_revisions")
        pub.sendMessage("request_update_all_functions")
        pub.sendMessage("request_update_all_requirements")
        pub.sendMessage("request_update_all_stakeholders")
        pub.sendMessage("request_update_all_hardware")
        pub.sendMessage("request_update_all_validation")



# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
import treelib

# RAMSTK Package Imports
from ramstk.db.base import BaseDatabase as BaseDatabase
from ramstk.exceptions import DataAccessError as DataAccessError
from ramstk.utilities import none_to_default as none_to_default

def do_clear_tree(tree: treelib.Tree) -> treelib.Tree: ...

class RAMSTKBaseRecord:
    def set_attributes(self, attributes) -> None: ...

class RAMSTKBaseTable:
    _db_id_colname: str
    _db_tablename: str
    _root: int
    _select_msg: str
    _tag: str
    _fkey: Any
    _pkey: Any
    _lst_id_columns: Any
    _parent_id: int
    _record: Any
    _revision_id: int
    dao: Any
    last_id: int
    pkey: str
    tree: Any
    do_get_new_record: Any
    def __init__(self, **kwargs: Dict[str, Any]) -> None: ...
    def do_connect(self, dao: BaseDatabase) -> None: ...
    def do_create_all_codes(self, prefix: str) -> None: ...
    def do_delete(self, node_id: int) -> None: ...
    def do_get_attributes(self, node_id: int, table: str = ...) -> None: ...
    def do_get_tree(self) -> None: ...
    def do_insert(self, attributes: Dict[str, Any]) -> None: ...
    def do_select(self, node_id: Any) -> Any: ...
    def do_select_all(self, attributes: Dict[str, Any]) -> None: ...
    def do_set_attributes(self, node_id: List, package: Dict[str, Any]) -> None: ...
    def do_set_attributes_all(self, attributes: Dict[str, Any]) -> None: ...
    def do_set_tree(self, tree: treelib.Tree) -> None: ...
    def do_update(self, node_id: int, table: str = ...) -> None: ...
    def do_update_all(self) -> None: ...

class RAMSTKBaseView:
    _root: int
    _tag: str
    _dic_load_functions: Any
    _dic_trees: Any
    _lst_modules: Any
    _revision_id: int
    dao: Any
    tree: Any
    def __init__(self, **kwargs: Dict[str, Any]) -> None: ...
    def do_set_tree(self, tree: treelib.Tree) -> None: ...
    def on_select_all(self) -> None: ...



# Standard Library Imports
from typing import Any, Tuple

def get_hazard_rate(
    shape: float, time: float, location: float = ..., scale: float = ...
) -> float: ...
def get_mtbf(shape: float, location: float = ..., scale: float = ...) -> float: ...
def get_survival(
    shape: float, time: float, location: float = ..., scale: float = ...
) -> float: ...
def do_fit(data: Any, **kwargs: Any) -> Tuple[float, float, float]: ...



# Standard Library Imports
from typing import List, TypeVar

# RAMSTK Package Imports
from ramstk.configuration import RAMSTKSiteConfiguration as RAMSTKSiteConfiguration
from ramstk.configuration import RAMSTKUserConfiguration as RAMSTKUserConfiguration
from ramstk.logger import RAMSTKLogManager as RAMSTKLogManager
from ramstk.views.gtk3 import Gdk as Gdk
from ramstk.views.gtk3 import GdkPixbuf as GdkPixbuf
from ramstk.views.gtk3 import GObject as GObject
from ramstk.views.gtk3 import Gtk as Gtk
from ramstk.views.gtk3 import _ as _
from ramstk.views.gtk3.assistants import CreateProject as CreateProject
from ramstk.views.gtk3.assistants import ExportProject as ExportProject
from ramstk.views.gtk3.assistants import ImportProject as ImportProject
from ramstk.views.gtk3.assistants import OpenProject as OpenProject
from ramstk.views.gtk3.books import RAMSTKListBook as RAMSTKListBook
from ramstk.views.gtk3.books import RAMSTKModuleBook as RAMSTKModuleBook
from ramstk.views.gtk3.books import RAMSTKWorkBook as RAMSTKWorkBook
from ramstk.views.gtk3.options import OptionsDialog as OptionsDialog
from ramstk.views.gtk3.preferences import PreferencesDialog as PreferencesDialog

Tconfiguration = TypeVar(
    "Tconfiguration", RAMSTKUserConfiguration, RAMSTKSiteConfiguration
)

def destroy(__widget: Gtk.Widget, __event: Gdk.Event = ...) -> None: ...

class RAMSTKDesktop(Gtk.Window):
    RAMSTK_SITE_CONFIGURATION: RAMSTKSiteConfiguration = ...
    RAMSTK_USER_CONFIGURATION: RAMSTKUserConfiguration = ...
    _logger: RAMSTKLogManager = ...
    _height: float = ...
    _n_screens: int = ...
    _width: float = ...
    menubar: Gtk.MenuBar = ...
    progressbar: Gtk.ProgressBar = ...
    statusbar: Gtk.Statusbar = ...
    toolbar: Gtk.Toolbar = ...
    icoStatus: Gtk.StatusIcon = ...
    nbkListBook: RAMSTKListBook = ...
    nbkModuleBook: RAMSTKModuleBook = ...
    nbkWorkBook: RAMSTKWorkBook = ...
    def __init__(
        self, configuration: Tconfiguration, logger: RAMSTKLogManager
    ) -> None: ...
    def _do_request_options_assistant(self, __widget: Gtk.ImageMenuItem) -> None: ...
    def _do_request_preferences_assistant(
        self, __widget: Gtk.ImageMenuItem
    ) -> None: ...
    def __make_menu(self) -> None: ...
    def __make_menu_edit(self) -> Gtk.MenuItem: ...
    def __make_menu_file(self) -> Gtk.MenuItem: ...
    def __make_menu_items(
        self, icons: List[str], labels: List[str]
    ) -> List[Gtk.ImageMenuItem]: ...
    def __make_menu_tools(self) -> Gtk.MenuItem: ...
    def __make_toolbuttons(
        self, icons: List[str], tooltips: List[str]
    ) -> List[Gtk.ToolButton]: ...
    def __make_toolbar(self) -> None: ...
    def __make_ui(self) -> None: ...
    def __set_callbacks(self) -> None: ...
    def __set_properties(self) -> None: ...
    @staticmethod
    def _do_request_close_project(__widget: Gtk.Widget) -> None: ...
    def _do_request_save_project(self, widget: Gtk.Widget, end: bool = ...) -> None: ...
    def _do_set_status(self, status: str) -> None: ...
    def _do_set_status_icon(self, connected: bool = ...) -> None: ...
    def _on_button_press(self, __book: object, event: Gdk.EventButton) -> None: ...
    def _on_request_open(self) -> None: ...
    def _on_select(self, title: str) -> None: ...
    @staticmethod
    def _on_window_state_event(
        window: Gtk.Window, event: Gdk.EventWindowState
    ) -> None: ...



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.program_status.panel.py is part of the RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Program Status Panels."""

# Standard Library Imports
from typing import Dict

# Third Party Imports
import pandas as pd

# noinspection PyPackageValidations,PyPackageRequirements
from matplotlib.patches import Ellipse
from pandas.plotting import register_matplotlib_converters
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import _
from ramstk.views.gtk3.widgets import RAMSTKPlotPanel

register_matplotlib_converters()


class ProgramStatusPlotPanel(RAMSTKPlotPanel):
    """Panel to display the Verification plan efforts."""

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field = "status_id"
    _select_msg = "selected_revision"
    _tag = "program_status"
    _title = _("Verification Plan Effort")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Burndown Curve panel."""
        super().__init__()

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.

        # Initialize public list instance attributes.
        self.lst_axis_labels = [_(""), _("Total Time [hours]")]
        self.lst_legend = [
            _("Minimum Expected Time"),
            _("Mean Expected Time"),
            _("Maximum Expected Time"),
            _("Actual Remaining Time"),
        ]

        # Initialize public scalar instance attributes.
        self.plot_title = _("Total Verification Effort")

        super().do_make_panel()

        # Subscribe to PyPubSub messages.
        pub.subscribe(self._do_load_panel, "succeed_calculate_verification_plan")

    def _do_load_panel(self, attributes: Dict[str, pd.DataFrame]) -> None:
        """Load the burndown curve with the planned and actual status.

        :param attributes: a dict containing a pandas DataFrames() for each of
            planned burndown, assessment dates/targets, and the actual
            progress.
        :return: None
        """
        self._do_load_plan(attributes["plan"])
        self._do_load_assessment_milestones(
            attributes["assessed"], attributes["plan"].loc[:, "upper"].max()
        )

        self.pltPlot.do_add_line(
            x_values=list(attributes["actual"].index),
            y_values=list(attributes["actual"].loc[:, "time"]),
            marker="o",
        )

        super().do_load_panel()

    def _do_load_assessment_milestones(
        self, assessed: pd.DataFrame, y_max: float
    ) -> None:
        """Add the reliability assessment milestones to the plot.

        This method will add a vertical line at all the dates identified as
        dates when a reliability assessment is due.  Annotated along side
        these markers are the reliability targets (lower, mean, upper) for that
        assessment date.

        :return: None
        :rtype: None
        """
        _y_max = max(1.0, y_max)

        for _date in list(assessed.index):
            self.pltPlot.axis.axvline(
                x=_date,
                ymin=0,
                ymax=1.05 * _y_max,
                color="k",
                linewidth=1.0,
                linestyle="-.",
            )
            self.pltPlot.axis.annotate(
                str(self.fmt.format(assessed.loc[pd.to_datetime(_date), "upper"]))
                + "\n"
                + str(self.fmt.format(assessed.loc[pd.to_datetime(_date), "mean"]))
                + "\n"
                + str(self.fmt.format(assessed.loc[pd.to_datetime(_date), "lower"])),
                xy=(_date, 0.9 * _y_max),
                xycoords="data",
                xytext=(-55, 0),
                textcoords="offset points",
                size=12,
                va="center",
                bbox=dict(boxstyle="round", fc="#E5E5E5", ec="None", alpha=0.5),
                arrowprops=dict(
                    arrowstyle="wedge,tail_width=1.",
                    fc="#E5E5E5",
                    ec="None",
                    alpha=0.5,
                    patchA=None,
                    patchB=Ellipse((2, -1), 0.5, 0.5),
                    relpos=(0.2, 0.5),
                ),
            )

    def _do_load_plan(self, plan: pd.DataFrame) -> None:
        """Load the verification plan burndown curve.

        :param plan: the pandas DataFrame() containing the planned task end
            dates and remaining hours of work (lower, mean, upper).
        :return: None
        :rtype: None
        """
        self.pltPlot.axis.cla()
        self.pltPlot.axis.grid(True, which="both")

        self.pltPlot.do_load_plot(
            **{
                "x_values": list(plan.index),
                "y_values": list(plan.loc[:, "lower"]),
                "plot_type": "date",
                "marker": "g--",
            }
        )
        self.pltPlot.do_load_plot(
            **{
                "x_values": list(plan.index),
                "y_values": list(plan.loc[:, "mean"]),
                "plot_type": "date",
                "marker": "b-",
            }
        )
        self.pltPlot.do_load_plot(
            **{
                "x_values": list(plan.index),
                "y_values": list(plan.loc[:, "upper"]),
                "plot_type": "date",
                "marker": "r--",
            }
        )



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.assistants.similaritem.py is part of The RAMSTK
#       Project
#
# All rights reserved.
# Copyright 2019 Doyle Rowland doyle.rowland <AT> reliaqual <DOT> com
"""The RASMTK Similar Item Analysis Assistants Module."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKCheckButton,
    RAMSTKDialog,
    RAMSTKEntry,
    RAMSTKLabel,
    RAMSTKTreeView,
)


class EditFunction(RAMSTKDialog):
    """Assistant for editing similar item functions."""

    def __init__(self, treeview: RAMSTKTreeView, **kwargs: Dict[str, Any]) -> None:
        """Initialize instance of the Similar Item Function Editor Assistant.

        :param treeview: the Similar Item Work View RAMSTKTreeView().
        :type treeview: :class:`ramstk.views.gtk3.RAMSTKTreeView`
        """
        _dlgparent = kwargs["dlgparent"]
        super().__init__(
            _("RAMSTK Similar Item Analysis User Function Editing Assistant"),
            dlgparent=_dlgparent,
        )

        # Initialize private dictionary attributes.

        # Initialize private list attributes.

        # Initialize private scalar attributes.

        # Initialize public dictionary attributes.

        # Initialize public list attributes.

        # Initialize public scalar attributes.
        self.chkApplyAll: RAMSTKCheckButton = RAMSTKCheckButton(
            label=_("Apply to all assemblies.")
        )
        self.txtFunction1: RAMSTKEntry = RAMSTKEntry()
        self.txtFunction2: RAMSTKEntry = RAMSTKEntry()
        self.txtFunction3: RAMSTKEntry = RAMSTKEntry()
        self.txtFunction4: RAMSTKEntry = RAMSTKEntry()
        self.txtFunction5: RAMSTKEntry = RAMSTKEntry()

        self.__make_ui()
        self._do_load_functions(treeview)

    def __make_ui(self) -> None:
        """Build the user interface.

        :return: None
        :rtype: None
        """
        self.set_default_size(610, -1)

        _label1: RAMSTKLabel = RAMSTKLabel(
            _(
                "You can define up to five functions.  "
                "You can use the system failure rate, "
                "selected assembly failure rate, the "
                "change factor, the user float, the "
                "user integer values, and results of "
                "other functions.\n\n \
        System hazard rate is hr_sys\n \
        Assembly hazard rate is hr\n \
        Change factor is pi[1-8]\n \
        User float is uf[1-3]\n \
        User integer is ui[1-3]\n \
        Function result is res[1-5]\n\n"
            )
        )
        _label1.do_set_properties(width=600, height=-1, wrap=True)
        _height1 = _label1.get_attribute("height")

        _label2: RAMSTKLabel = RAMSTKLabel(
            _(
                "For example, pi1*pi2+pi3, multiplies "
                "the first two change factors and "
                "adds the value to the third change "
                "factor.\n\n"
            )
        )
        _label2.do_set_properties(width=600, height=-1, wrap=True)
        _height2 = _label2.get_attribute("height")

        # Build the dialog assistant.
        _fixed = Gtk.Fixed()

        _y_pos = 10
        _fixed.put(_label1, 5, _y_pos)
        _y_pos += _height1 + 50
        _fixed.put(_label2, 5, _y_pos)
        _y_pos += _height2 + 30

        _separator = Gtk.HSeparator()
        _separator.props.width_request = 600
        _fixed.put(_separator, 5, _y_pos)
        _y_pos += 20

        _label = RAMSTKLabel(_("User function 1:"))
        _fixed.put(_label, 5, _y_pos)
        _fixed.put(self.txtFunction1, 195, _y_pos)
        _y_pos += 30

        _label = RAMSTKLabel(_("User function 2:"))
        _fixed.put(_label, 5, _y_pos)
        _fixed.put(self.txtFunction2, 195, _y_pos)
        _y_pos += 30

        _label = RAMSTKLabel(_("User function 3:"))
        _fixed.put(_label, 5, _y_pos)
        _fixed.put(self.txtFunction3, 195, _y_pos)
        _y_pos += 30

        _label = RAMSTKLabel(_("User function 4:"))
        _fixed.put(_label, 5, _y_pos)
        _fixed.put(self.txtFunction4, 195, _y_pos)
        _y_pos += 30

        _label = RAMSTKLabel(_("User function 5:"))
        _fixed.put(_label, 5, _y_pos)
        _fixed.put(self.txtFunction5, 195, _y_pos)
        _y_pos += 30

        _fixed.put(self.chkApplyAll, 5, _y_pos)

        _fixed.show_all()

        self.vbox.pack_start(_fixed, True, True, 0)

    def do_set_functions(self, treeview: RAMSTKTreeView) -> List[str]:
        """Set the user-defined functions.

        :return: functions; a tuple of the five user-defined functions.
        :rtype: list
        """
        (_model, _row) = treeview.get_selection().get_selected()
        if self.chkApplyAll.get_active():
            _row = _model.get_iter_first()
            while _row is not None:
                _hardware_id = _model.get_value(_row, 1)
                self._on_set_function(_hardware_id)
                _row = _model.iter_next(_row)
        else:
            _hardware_id = _model.get_value(_row, 1)
            self._on_set_function(_hardware_id)

        return [
            str(self.txtFunction1.get_text()),
            str(self.txtFunction2.get_text()),
            str(self.txtFunction3.get_text()),
            str(self.txtFunction4.get_text()),
            str(self.txtFunction5.get_text()),
        ]

    def _cancel(self, __button: Gtk.Button) -> None:
        """Destroy the assistant when the 'Cancel' button is pressed.

        :param Gtk.Button __button: the Gtk.Button() that called this method.
        :return: None
        :rtype: None
        """
        self.destroy()

    def _do_load_functions(self, treeview: RAMSTKTreeView) -> None:
        """Load any existing user-defined functions.

        :param treeview: the Similar Item Work View RAMSTKTreeView().
        :type treeview: :class:`ramstk.gui.gtk.TreeView.RAMSTKTreeView`
        :return: None
        :rtype: None
        """
        (_model, _row) = treeview.get_selection().get_selected()
        try:
            self.txtFunction1.set_text(_model.get_value(_row, 30))
        except TypeError:
            self.txtFunction1.set_text("")
        try:
            self.txtFunction2.set_text(_model.get_value(_row, 31))
        except TypeError:
            self.txtFunction2.set_text("")
        try:
            self.txtFunction3.set_text(_model.get_value(_row, 32))
        except TypeError:
            self.txtFunction3.set_text("")
        try:
            self.txtFunction4.set_text(_model.get_value(_row, 33))
        except TypeError:
            self.txtFunction4.set_text("")
        try:
            self.txtFunction5.set_text(_model.get_value(_row, 34))
        except TypeError:
            self.txtFunction5.set_text("")

    def _on_set_function(self, hardware_id: int) -> None:
        """Send PyPubSub messages to update the similar item attributes.

        :param hardware_id: the Hardware ID for the Similar Item to update.
        :return: None
        :rtype: None
        """
        pub.sendMessage(
            "wvw_editing_similar_item",
            node_id=[hardware_id, -1],
            package={"function_1": str(self.txtFunction1.get_text())},
        )
        pub.sendMessage(
            "wvw_editing_similar_item",
            node_id=[hardware_id, -1],
            package={"function_2": str(self.txtFunction2.get_text())},
        )
        pub.sendMessage(
            "wvw_editing_similar_item",
            node_id=[hardware_id, -1],
            package={"function_3": str(self.txtFunction3.get_text())},
        )
        pub.sendMessage(
            "wvw_editing_similar_item",
            node_id=[hardware_id, -1],
            package={"function_4": str(self.txtFunction4.get_text())},
        )
        pub.sendMessage(
            "wvw_editing_similar_item",
            node_id=[hardware_id, -1],
            package={"function_5": str(self.txtFunction5.get_text())},
        )



# -*- coding: utf-8 -*-
#
#       ramstk.views.gtk3.design_electric.panel.py is part of The RAMSTK Project
#
# All rights reserved.
# Copyright since 2007 Doyle "weibullguy" Rowland doyle.rowland <AT> reliaqual <DOT> com
"""GTK3 Design Electric Panels."""

# Standard Library Imports
from typing import Any, Dict, List

# Third Party Imports
from pubsub import pub

# RAMSTK Package Imports
from ramstk.views.gtk3 import Gtk, _
from ramstk.views.gtk3.widgets import (
    RAMSTKCheckButton,
    RAMSTKComboBox,
    RAMSTKEntry,
    RAMSTKFixedPanel,
    RAMSTKPlot,
    RAMSTKScrolledWindow,
    RAMSTKTextView,
)


class DesignElectricEnvironmentalInputPanel(RAMSTKFixedPanel):
    """Panel to display environmental data about the selected hardware item.

    The widgets of a Design Electric environmental input panel are:

    :ivar cmbActiveEnviron: the RAMSTKComboBox() used to select the active (operating)
        environment for the selected hardware item.  These are the MIL-HDBK-217
        environments.
    :ivar cmbDormantEnviron: the RAMSTKComboBox() used to select the dormant (storage)
        environment for the selected hardware item.  These are the MIL-HDBK-338
        environments.
    :ivar txtActiveTemp: the RAMSTKEntry() used to input and display the active
        (operating) temperature of the selected hardware item.
    :ivar txtDormantTemp: the RAMSTKEntry() used to input and display the dormant
        (storage) temperature of the selected hardware item.
    :ivar txtDutyCycle: the RAMSTKEntry() used to input and display the duty cycle of
        the selected hardware item.
    :ivar txtMissionTime: the RAMSTKEntry() used to input and display the mission
        time of the selected hardware item.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "succeed_get_design_electric_attributes"
    _tag: str = "design_electric"
    _title: str = _("Hardware Environmental Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Environmental Input panel."""
        super().__init__()

        # Initialize widgets.
        self.cmbActiveEnviron: RAMSTKComboBox = RAMSTKComboBox()
        self.cmbDormantEnviron: RAMSTKComboBox = RAMSTKComboBox()
        self.txtActiveTemp: RAMSTKEntry = RAMSTKEntry()
        self.txtDormantTemp: RAMSTKEntry = RAMSTKEntry()
        self.txtDutyCycle: RAMSTKEntry = RAMSTKEntry()
        self.txtMissionTime: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "environment_active_id": [
                12,
                self.cmbActiveEnviron,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The operating environment for the hardware item."),
                    "width": 200,
                },
                _("Active Environment:"),
                "gint",
            ],
            "temperature_active": [
                37,
                self.txtActiveTemp,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The ambient temperature in the operating environment."
                    ),
                    "width": 125,
                },
                _("Active Temperature (\u00B0C):"),
                "gfloat",
            ],
            "environment_dormant_id": [
                13,
                self.cmbDormantEnviron,
                "changed",
                super().on_changed_combo,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The storage environment for the hardware item."),
                    "width": 200,
                },
                _("Dormant Environment:"),
                "gint",
            ],
            "temperature_dormant": [
                39,
                self.txtDormantTemp,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The ambient temperature in the storage environment."),
                    "width": 125,
                },
                _("Dormant Temperature (\u00B0C):"),
                "gfloat",
            ],
            "mission_time": [
                14,
                self.txtMissionTime,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                1.0,
                {
                    "tooltip": _("The mission time of the selected hardware item."),
                    "width": 125,
                },
                _("Mission Time:"),
                "gfloat",
            ],
            "duty_cycle": [
                9,
                self.txtDutyCycle,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                100.0,
                {
                    "tooltip": _("The duty cycle of the selected hardware item."),
                    "width": 125,
                },
                _("Duty Cycle:"),
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        # Make a fixed type panel.
        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.

    def do_load_environment_active(self, environments: List[str]) -> None:
        """Load the active environments RAMSTKComboBox().

        :param environments: the list of active environments.
        :return: None
        """
        self.cmbActiveEnviron.do_load_combo(entries=environments)

    def do_load_environment_dormant(self, environments: List[str]) -> None:
        """Load the dormant environments RAMSTKComboBox().

        :param environments: the list of dormant environments.
        :return: None
        """
        self.cmbDormantEnviron.do_load_combo(entries=environments)


class DesignElectricStressInputPanel(RAMSTKFixedPanel):
    """Panel to display environmental data about the selected Hardware item.

    The widgets of a Design Electric electrical stress input panel are:

    :ivar txtTemperatureRatedMin: the RAMSTKEntry() used to input and display the
        minimum rated temperature of the selected hardware item.
    :ivar txtTemperatureKnee: the RAMSTKEntry() used to input and display the knee
        temperature of the selected hardware item.
    :ivar txtTemperatureRatedMax: the RAMSTKEntry() used to input and display the
        maximum rated temperature of the selected hardware item.
    :ivar txtCurrentRated: the RAMSTKEntry() used to input and display the rated
        current of the selected hardware item.
    :ivar txtCurrentOperating: the RAMSTKEntry() used to input and display the
        operating current of the selected hardware item.
    :ivar txtPowerRated: the RAMSTKEntry() used to input and display the rated power of
        the selected hardware item.
    :ivar txtPowerOperating: the RAMSTKEntry() used to input and display the
        operating power of the selected hardware item.
    :ivar txtVoltageRated: the RAMSTKEntry() used to input and display the rated
        voltage of the selected hardware item.
    :ivar txtVoltageAC: the RAMSTKEntry() used to input and display the operating AC
        voltage of the selected hardware item.
    :ivar txtVoltageDC: the RAMSTKEntry() used to input and display the operating DC
        voltage of the selected hardware item.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "succeed_get_design_electric_attributes"
    _tag: str = "design_electric"
    _title: str = _("Hardware Thermal &amp; Electrical Stress Inputs")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Stress Input panel."""
        super().__init__()

        # Initialize widgets.
        self.txtTemperatureRatedMin: RAMSTKEntry = RAMSTKEntry()
        self.txtTemperatureKnee: RAMSTKEntry = RAMSTKEntry()
        self.txtTemperatureRatedMax: RAMSTKEntry = RAMSTKEntry()
        self.txtCurrentRated: RAMSTKEntry = RAMSTKEntry()
        self.txtCurrentOperating: RAMSTKEntry = RAMSTKEntry()
        self.txtPowerRated: RAMSTKEntry = RAMSTKEntry()
        self.txtPowerOperating: RAMSTKEntry = RAMSTKEntry()
        self.txtVoltageRated: RAMSTKEntry = RAMSTKEntry()
        self.txtVoltageAC: RAMSTKEntry = RAMSTKEntry()
        self.txtVoltageDC: RAMSTKEntry = RAMSTKEntry()

        # Initialize private dict instance attributes.

        # Initialize private list instance attributes.

        # Initialize private scalar instance attributes.

        # Initialize public dict instance attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "temperature_rated_min": [
                45,
                self.txtTemperatureRatedMin,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                25.0,
                {
                    "tooltip": _(
                        "The minimum rated temperature (in \u00B0C) of the hardware "
                        "item."
                    ),
                    "width": 125,
                },
                _("Minimum Rated Temperature (\u00B0C):"),
                "gfloat",
            ],
            "temperature_knee": [
                43,
                self.txtTemperatureKnee,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                25.0,
                {
                    "tooltip": _(
                        "The break temperature (in \u00B0C) of the hardware item "
                        "beyond which it must be derated."
                    ),
                    "width": 125,
                },
                _("Knee Temperature (\u00B0C):"),
                "gfloat",
            ],
            "temperature_rated_max": [
                44,
                self.txtTemperatureRatedMax,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                25.0,
                {
                    "tooltip": _(
                        "The maximum rated temperature (in \u00B0C) of the hardware "
                        "item."
                    ),
                    "width": 125,
                },
                _("Maximum Rated Temperature (\u00B0C):"),
                "gfloat",
            ],
            "current_rated": [
                11,
                self.txtCurrentRated,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The rated current (in A) of the hardware item."),
                    "width": 125,
                },
                _("Rated Current (A):"),
                "gfloat",
            ],
            "current_operating": [
                10,
                self.txtCurrentOperating,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The operating current (in A) of the hardware item."),
                    "width": 200,
                },
                _("Operating Current (A):"),
                "gfloat",
            ],
            "power_rated": [
                32,
                self.txtPowerRated,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The rated power (in W) of the hardware item."),
                    "width": 125,
                },
                _("Rated Power (W):"),
                "gfloat",
            ],
            "power_operating": [
                31,
                self.txtPowerOperating,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The operating power (in W) of the hardware item."),
                    "width": 200,
                },
                _("Operating Power (W):"),
                "gfloat",
            ],
            "voltage_rated": [
                52,
                self.txtVoltageRated,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _("The rated voltage (in V) of the hardware item."),
                    "width": 125,
                },
                _("Rated Voltage (V):"),
                "gfloat",
            ],
            "voltage_ac_operating": [
                49,
                self.txtVoltageAC,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The operating ac voltage (in V) of the hardware item."
                    ),
                    "width": 125,
                },
                _("Operating ac Voltage (V):"),
                "gfloat",
            ],
            "voltage_dc_operating": [
                50,
                self.txtVoltageDC,
                "changed",
                super().on_changed_entry,
                f"wvw_editing_{self._tag}",
                0.0,
                {
                    "tooltip": _(
                        "The operating DC voltage (in V) of the hardware item."
                    ),
                    "width": 125,
                },
                _("Operating DC Voltage (V):"),
                "gfloat",
            ],
        }

        # Initialize public list instance attributes.

        # Initialize public scalar instance attributes.

        # Make a fixed type panel.
        super().do_set_properties()
        super().do_make_panel()
        super().do_set_callbacks()

        # Subscribe to PyPubSub messages.


class DesignElectricStressResultPanel(RAMSTKFixedPanel):
    """Display Hardware stress results attribute data in the RAMSTK Work Book.

    The widgets of a Design Electric electrical stress results panel are:

    :ivar chkOverstress: the RAMSTKCheckButton() used to indicate whether the selected
        hardware item is electrically or thermally overstressed.
    :ivar pltPlot: the RAMSTKPlot() used to display the derating curve and operating
        point of the selected component.
    :ivar txtCurrentRatio: the RAMSTKEntry() used to display the ratio of operating
        to rated current for the selected component.
    :ivar txtPowerRatio: the RAMSTKEntry() used to display the ratio of operating to
        rated power for the selected component.
    :ivar txtVoltageRatio: the RAMSTKEntry() used to display the ratio of operating
        to rated voltage for the selected component.
    :ivar txtReason: the RAMSTKTextView() used to display the reason(s), if any,
        the selected hardware item is overstressed.

    The attributes of a Design Electricl electrical stress results panel are:

    :ivar list _lst_derate_criteria: the derating criteria for the selected component.
    :ivar _category_id: the hardware category ID of the selected component.
    :ivar _part_number: the part number of the selected component.
    :ivar _ref_des: the reference designator of the selected component.
    """

    # Define private dictionary class attributes.

    # Define private list class attributes.

    # Define private scalar class attributes.
    _record_field: str = "hardware_id"
    _select_msg: str = "succeed_get_design_electric_attributes"
    _tag: str = "design_electric"
    _title: str = _("Hardware Thermal &amp; Electrical Stress Summary")

    # Define public dictionary class attributes.

    # Define public list class attributes.

    # Define public scalar class attributes.

    def __init__(self) -> None:
        """Initialize an instance of the Hardware stress result view."""
        super().__init__()

        # Initialize widgets.
        self.chkOverstress: RAMSTKCheckButton = RAMSTKCheckButton(
            label=_("Overstressed")
        )
        self.pltPlot: RAMSTKPlot = RAMSTKPlot()
        self.txtCurrentRatio: RAMSTKEntry = RAMSTKEntry()
        self.txtPowerRatio: RAMSTKEntry = RAMSTKEntry()
        self.txtVoltageRatio: RAMSTKEntry = RAMSTKEntry()
        self.txtReason: RAMSTKTextView = RAMSTKTextView(Gtk.TextBuffer())

        # Initialize private dictionary attributes.

        # Initialize private list attributes.
        self._lst_derate_criteria: List[List[float]] = [
            [0.6, 0.6, 0.0],
            [0.9, 0.9, 0.0],
        ]

        # Initialize private scalar attributes.
        self._category_id: int = 0
        self._part_number: str = ""
        self._ref_des: str = ""

        # Initialize public dictionary attributes.
        self.dic_attribute_widget_map: Dict[str, List[Any]] = {
            "current_ratio": [
                12,
                self.txtCurrentRatio,
                "",
                None,
                "",
                0.5,
                {
                    "bold": True,
                    "editable": False,
                    "tooltip": _(
                        "The ratio of operating current to rated current for the "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Current Ratio:"),
                "gfloat",
            ],
            "power_ratio": [
                33,
                self.txtPowerRatio,
                "",
                None,
                "",
                0.5,
                {
                    "bold": True,
                    "editable": False,
                    "tooltip": _(
                        "The ratio of operating power to rated power for the "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Power Ratio:"),
                "gfloat",
            ],
            "voltage_ratio": [
                53,
                self.txtVoltageRatio,
                "",
                None,
                "",
                0.5,
                {
                    "bold": True,
                    "editable": False,
                    "tooltip": _(
                        "The ratio of operating voltage to rated voltage for the "
                        "hardware item."
                    ),
                    "width": 125,
                },
                _("Voltage Ratio:"),
                "gfloat",
            ],
            "overstressed": [
                29,
                self.chkOverstress,
                "",
                None,
                "",
                0,
                {
                    "tooltip": _(
                        "Indicates whether or not the selected hardware item is "
                        "overstressed."
                    ),
                    "width": 125,
                },
                "",
                "gint",
            ],
            "reason": [
                34,
                self.txtReason,
                "",
                None,
                "",
                "",
                {
                    "bold": True,
                    "editable": False,
                    "tooltip": _(
                        "The reason(s) the selected hardware item is overstressed."
                    ),
                    "width": 350,
                },
                _("Overstress Reason:"),
                "gchararray",
            ],
        }

        # Initialize public list attributes.

        # Initialize public scalar attributes.

        super().do_set_properties()
        super().do_make_panel()
        self.__make_ui()

        # Subscribe to PyPubSub messages.
        pub.subscribe(
            self._do_set_hardware_attributes,
            "succeed_get_hardware_attributes",
        )

    def _do_load_derating_curve(
        self, attributes: Dict[str, Any], stress: str = "voltage"
    ) -> None:
        """Load the benign and harsh environment derating curves.

        :return: None
        :rtype: None
        """
        self.pltPlot.axis.cla()

        _x = [
            float(attributes["temperature_rated_min"]),
            float(attributes["temperature_knee"]),
            float(attributes["temperature_rated_max"]),
        ]

        self.pltPlot.axis.grid(True, which="both")

        self.pltPlot.do_load_plot(
            x_values=_x, y_values=self._lst_derate_criteria[0], marker="r.-"
        )

        self.pltPlot.do_load_plot(
            x_values=_x, y_values=self._lst_derate_criteria[1], marker="b.-"
        )

        self.pltPlot.do_load_plot(
            x_values=[attributes["temperature_active"]],
            y_values=[attributes[f"{stress}_ratio"]],
            marker="go",
        )

        self.pltPlot.do_make_title(
            _(
                f"{stress.title()} Derating Curve for {self._part_number} at "
                f"{self._ref_des}"
            ),
            fontsize=12,
        )

        self.pltPlot.do_make_legend(
            (
                _("Harsh Environment"),
                _("Mild Environment"),
                _(f"{stress.title()} Operating Point"),
            )
        )

        self.pltPlot.do_make_labels(
            _("Temperature (\u2070C)"), x_pos=0, y_pos=-0.2, fontsize=10
        )
        self.pltPlot.do_make_labels(
            _(f"{stress.title()} Ratio"),
            x_pos=-1,
            y_pos=0,
            set_x=False,
            fontsize=10,
        )

        self.pltPlot.figure.canvas.draw()

    def _do_load_entries(self, attributes: Dict[str, Any]) -> None:
        """Load the stress results page widgets.

        :param attributes: the attribute dict for the selected Hardware.
        :return: None
        :rtype: None
        """
        self.txtCurrentRatio.do_update(
            str(self.fmt.format(attributes["current_ratio"] or 0.0))
        )
        self.txtPowerRatio.do_update(
            str(self.fmt.format(attributes["power_ratio"] or 0.0))
        )
        self.txtVoltageRatio.do_update(
            str(self.fmt.format(attributes["voltage_ratio"] or 0.0))
        )
        self.chkOverstress.set_active(attributes["overstress"] or 0)
        self.txtReason.do_update(attributes["reason"] or "")

        if self._category_id in [2, 4]:
            self._do_load_derating_curve(attributes, stress="voltage")
        elif self._category_id == 3:
            self._do_load_derating_curve(attributes, stress="power")
        elif self._category_id in [6, 7]:
            self._do_load_derating_curve(attributes, stress="current")

    def _do_set_hardware_attributes(self, attributes: Dict[str, Any]) -> None:
        """Set the attributes when the reliability attributes are retrieved.

        :param attributes: the dict of reliability attributes.
        :return: None
        :rtype: None
        """
        if attributes["hardware_id"] == self._record_id:
            self._category_id = attributes["category_id"]
            self._part_number = attributes["part_number"]
            self._ref_des = attributes["ref_des"]

    def __make_ui(self) -> None:
        """Make the Hardware stress results page.

        :return: None
        :rtype: None
        """
        _scrollwindow: RAMSTKScrolledWindow = self.get_child()
        self.remove(self.get_child())

        _hpaned: Gtk.HPaned = Gtk.HPaned()
        self.add(_hpaned)

        _hpaned.pack1(_scrollwindow, False, False)

        _scrollwindow = RAMSTKScrolledWindow(self.pltPlot.canvas)
        _hpaned.pack2(_scrollwindow, False, False)

        self.show_all()
